/// <reference types="node" />
import type { BackoffStrategy } from 'backoff';
import { AxiosRequestConfig } from 'axios';
import S3 from './S3';
import { Dump } from './Dump';
import MediaWiki from './MediaWiki';
interface DownloaderOpts {
    mw: MediaWiki;
    uaString: string;
    speed: number;
    reqTimeout: number;
    noLocalParserFallback: boolean;
    forceLocalParser: boolean;
    optimisationCacheUrl: string;
    s3?: S3;
    webp: boolean;
    backoffOptions?: BackoffOptions;
}
interface BackoffOptions {
    strategy: BackoffStrategy;
    failAfter: number;
    retryIf: (error?: any) => boolean;
    backoffHandler: (number: number, delay: number, error?: any) => void;
}
export interface MWCapabilities {
    apiAvailable: boolean;
    veApiAvailable: boolean;
    coordinatesAvailable: boolean;
    desktopRestApiAvailable: boolean;
    mobileRestApiAvailable: boolean;
}
declare class Downloader {
    readonly mw: MediaWiki;
    loginCookie: string;
    readonly speed: number;
    baseUrl: string;
    baseUrlForMainPage: string;
    cssDependenceUrls: KVS<boolean>;
    readonly webp: boolean;
    private readonly uaString;
    private activeRequests;
    private maxActiveRequests;
    private readonly requestTimeout;
    private readonly noLocalParserFallback;
    private readonly forceLocalParser;
    private readonly urlPartCache;
    private readonly backoffOptions;
    private readonly optimisationCacheUrl;
    private s3;
    private mwCapabilities;
    arrayBufferRequestOptions: AxiosRequestConfig;
    jsonRequestOptions: AxiosRequestConfig;
    streamRequestOptions: AxiosRequestConfig;
    constructor({ mw, uaString, speed, reqTimeout, noLocalParserFallback, forceLocalParser: forceLocalParser, optimisationCacheUrl, s3, webp, backoffOptions }: DownloaderOpts);
    serializeUrl(url: string): string;
    deserializeUrl(url: string): string;
    setBaseUrls(): Promise<void>;
    checkApiAvailabilty(url: string): Promise<boolean>;
    checkCapabilities(testArticleId?: string): Promise<void>;
    removeEtagWeakPrefix(etag: string): string;
    initLocalServices(): Promise<void>;
    query(query: string): KVS<any>;
    getArticleDetailsIds(articleIds: string[], shouldGetThumbnail?: boolean): Promise<QueryMwRet>;
    getArticleDetailsNS(ns: number, gapcontinue?: string): Promise<{
        gapContinue: string;
        articleDetails: QueryMwRet;
    }>;
    getArticle(articleId: string, dump: Dump): Promise<RenderedArticle[]>;
    getJSON<T>(_url: string): Promise<T>;
    downloadContent(_url: string): Promise<{
        content: Buffer | string;
        responseHeaders: any;
    }>;
    canGetUrl(url: string): Promise<boolean>;
    private getArticleUrl;
    private stripNonContinuedProps;
    private static handleMWWarningsAndErrors;
    private getArticleQueryOpts;
    private setArticleSubCategories;
    private claimRequest;
    private releaseRequest;
    private getJSONCb;
    private getCompressedBody;
    private getContentCb;
    private downloadImage;
    private errHandler;
    private getSubCategories;
    private backoffCall;
}
export default Downloader;
