"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const urlParser = __importStar(require("url"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const backoff = __importStar(require("backoff"));
const imagemin = __importStar(require("imagemin"));
const service_runner_1 = __importDefault(require("service-runner"));
const imagemin_advpng_1 = __importDefault(require("imagemin-advpng"));
const axios_1 = __importDefault(require("axios"));
const imagemin_pngquant_1 = __importDefault(require("imagemin-pngquant"));
const imagemin_gifsicle_1 = __importDefault(require("imagemin-gifsicle"));
const imagemin_jpegoptim_1 = __importDefault(require("imagemin-jpegoptim"));
const imagemin_webp_1 = __importDefault(require("imagemin-webp"));
const sharp_1 = __importDefault(require("sharp"));
const util_1 = require("./util");
const Logger_1 = __importDefault(require("./Logger"));
const imageminOptions = new Map();
imageminOptions.set('default', new Map());
imageminOptions.set('webp', new Map());
imageminOptions.get('default').set('image/png', {
    plugins: [
        imagemin_pngquant_1.default({ speed: 3, strip: true, dithering: 0 }),
        imagemin_advpng_1.default({ optimizationLevel: 4, iterations: 5 }),
    ],
});
imageminOptions.get('default').set('image/jpeg', {
    plugins: [
        imagemin_jpegoptim_1.default({ max: 60, stripAll: true }),
    ],
});
imageminOptions.get('default').set('image/gif', {
    plugins: [
        imagemin_gifsicle_1.default({ optimizationLevel: 3, colors: 64 }),
    ],
});
imageminOptions.get('webp').set('image/png', {
    plugins: [
        imagemin_webp_1.default({ quality: 50, method: 6 })
    ],
});
imageminOptions.get('webp').set('image/jpeg', {
    plugins: [
        imagemin_webp_1.default({ quality: 50, method: 6 })
    ],
});
class Downloader {
    constructor({ mw, uaString, speed, reqTimeout, noLocalParserFallback, forceLocalParser: forceLocalParser, optimisationCacheUrl, s3, webp, backoffOptions }) {
        this.loginCookie = '';
        this.cssDependenceUrls = {};
        this.webp = false;
        this.activeRequests = 0;
        this.maxActiveRequests = 1;
        this.noLocalParserFallback = false;
        this.forceLocalParser = false;
        this.urlPartCache = {};
        this.getJSONCb = (url, handler) => {
            Logger_1.default.info(`Getting JSON from [${url}]`);
            axios_1.default.get(url, this.jsonRequestOptions)
                .then((a) => handler(null, a.data), handler)
                .catch((err) => {
                try {
                    if (err.response && err.response.status === 429) {
                        Logger_1.default.log(`Received a [status=429], slowing down`);
                        const newMaxActiveRequests = Math.max(Math.ceil(this.maxActiveRequests * 0.9), 1);
                        Logger_1.default.log(`Setting maxActiveRequests from [${this.maxActiveRequests}] to [${newMaxActiveRequests}]`);
                        this.maxActiveRequests = newMaxActiveRequests;
                        return this.getJSONCb(url, handler);
                    }
                    else if (err.response && err.response.status === 404) {
                        handler(err);
                    }
                }
                catch (a) {
                    Logger_1.default.log('ERR', err);
                    handler(err);
                }
            });
        };
        this.getContentCb = (url, handler) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info(`Downloading [${url}]`);
            try {
                if (this.optimisationCacheUrl && util_1.isImageUrl(url)) {
                    this.downloadImage(url, handler);
                }
                else {
                    const resp = yield axios_1.default(url, this.arrayBufferRequestOptions);
                    yield this.getCompressedBody(resp);
                    handler(null, {
                        responseHeaders: resp.headers,
                        content: resp.data,
                    });
                }
            }
            catch (err) {
                try {
                    this.errHandler(err, url, handler);
                }
                catch (a) {
                    handler(err);
                }
            }
        });
        this.mw = mw;
        this.uaString = uaString;
        this.speed = speed;
        this.maxActiveRequests = speed * 10;
        this.requestTimeout = reqTimeout;
        this.loginCookie = '';
        this.noLocalParserFallback = noLocalParserFallback;
        this.forceLocalParser = forceLocalParser;
        this.optimisationCacheUrl = optimisationCacheUrl;
        this.webp = webp;
        this.s3 = s3;
        this.mwCapabilities = {
            apiAvailable: false,
            veApiAvailable: false,
            coordinatesAvailable: true,
            desktopRestApiAvailable: false,
            mobileRestApiAvailable: false
        };
        this.backoffOptions = Object.assign({ strategy: new backoff.ExponentialStrategy(), failAfter: 7, retryIf: (err) => { var _a; return err.code === 'ECONNABORTED' || ![400, 403, 404].includes((_a = err.response) === null || _a === void 0 ? void 0 : _a.status); }, backoffHandler: (number, delay) => {
                Logger_1.default.info(`[backoff] #${number} after ${delay} ms`);
            } }, backoffOptions);
        this.arrayBufferRequestOptions = {
            headers: {
                'cache-control': 'public, max-stale=86400',
                'user-agent': this.uaString,
                'cookie': this.loginCookie,
            },
            responseType: 'arraybuffer',
            timeout: this.requestTimeout,
            method: 'GET',
            validateStatus(status) { return (status >= 200 && status < 300) || status === 304; }
        };
        this.jsonRequestOptions = {
            headers: {
                'accept': 'application/json',
                'cache-control': 'public, max-stale=86400',
                'accept-encoding': 'gzip, deflate',
                'user-agent': this.uaString,
                'cookie': this.loginCookie,
            },
            responseType: 'json',
            timeout: this.requestTimeout,
            method: 'GET'
        };
        this.streamRequestOptions = {
            headers: {
                'accept': 'application/octet-stream',
                'cache-control': 'public, max-stale=86400',
                'accept-encoding': 'gzip, deflate',
                'user-agent': this.uaString,
                'cookie': this.loginCookie,
            },
            responseType: 'stream',
            timeout: this.requestTimeout,
            method: 'GET'
        };
    }
    serializeUrl(url) {
        const { path } = urlParser.parse(url);
        const cacheablePart = url.replace(path, '');
        const cacheEntry = Object.entries(this.urlPartCache).find(([cacheId, value]) => value === cacheablePart);
        let cacheKey;
        if (!cacheEntry) {
            const cacheId = String(Object.keys(this.urlPartCache).length + 1);
            this.urlPartCache[cacheId] = cacheablePart;
            cacheKey = `_${cacheId}_`;
        }
        else {
            cacheKey = `_${cacheEntry[0]}_`;
        }
        return `${cacheKey}${path}`;
    }
    deserializeUrl(url) {
        if (!url.startsWith('_'))
            return url;
        const [, cacheId, ...pathParts] = url.split('_');
        const path = pathParts.join('_');
        const cachedPart = this.urlPartCache[cacheId];
        return `${cachedPart}${path}`;
    }
    setBaseUrls() {
        return __awaiter(this, void 0, void 0, function* () {
            this.baseUrl = this.mwCapabilities.mobileRestApiAvailable ? this.mw.mobileRestApiUrl.href :
                this.mwCapabilities.desktopRestApiAvailable ? this.mw.desktopRestApiUrl.href :
                    this.mwCapabilities.veApiAvailable ? this.mw.veApiUrl.href :
                        !this.noLocalParserFallback && this.mwCapabilities.apiAvailable ? `http://localhost:6927/${this.mw.webUrl.hostname}/v1/page/mobile-sections/` :
                            undefined;
            this.baseUrlForMainPage = this.mwCapabilities.desktopRestApiAvailable ? this.mw.desktopRestApiUrl.href :
                this.mwCapabilities.veApiAvailable ? this.mw.veApiUrl.href :
                    !this.noLocalParserFallback && this.mwCapabilities.apiAvailable ? `http://localhost:8000/${this.mw.webUrl.hostname}/v3/page/pagebundle/` :
                        undefined;
            Logger_1.default.log('Base Url: ', this.baseUrl);
            Logger_1.default.log('Base Url for Main Page: ', this.baseUrlForMainPage);
            if (!this.baseUrl || !this.baseUrlForMainPage)
                throw new Error(`Unable to find appropriate API end-point to retrieve article HTML`);
            // TODO: This should not really be there, and not like this
            if (RegExp('.*http\:\/\/localhost.*').test(this.baseUrl + this.baseUrlForMainPage))
                yield this.initLocalServices();
        });
    }
    checkApiAvailabilty(url) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const resp = yield axios_1.default.get(url, { headers: { cookie: this.loginCookie } });
                // Check for hostname is for domain name in cases of redirects.
                return resp.status === 200
                    && !resp.headers['mediawiki-api-error']
                    && path.dirname(url) === path.dirname(resp.request.res.responseUrl);
            }
            catch (err) {
                return false;
            }
        });
    }
    checkCapabilities(testArticleId = 'MediaWiki:Sidebar') {
        return __awaiter(this, void 0, void 0, function* () {
            // By default check all API's responses and set the capabilities
            // accordingly. We need to set a default page (always there because
            // installed per default) to request the REST API, otherwise it would
            // fail the check.
            this.mwCapabilities.mobileRestApiAvailable = yield this.checkApiAvailabilty(this.mw.getMobileRestApiArticleUrl(testArticleId));
            this.mwCapabilities.desktopRestApiAvailable = yield this.checkApiAvailabilty(this.mw.getDesktopRestApiArticleUrl(testArticleId));
            this.mwCapabilities.veApiAvailable = yield this.checkApiAvailabilty(this.mw.getVeApiArticleUrl(testArticleId));
            this.mwCapabilities.apiAvailable = yield this.checkApiAvailabilty(this.mw.apiUrl.href);
            // Coordinate fetching
            const reqOpts = util_1.objToQueryString(Object.assign({}, this.getArticleQueryOpts()));
            const resp = yield this.getJSON(`${this.mw.apiUrl.href}${reqOpts}`);
            const isCoordinateWarning = resp.warnings && resp.warnings.query && (resp.warnings.query['*'] || '').includes('coordinates');
            if (isCoordinateWarning) {
                Logger_1.default.info(`Coordinates not available on this wiki`);
                this.mwCapabilities.coordinatesAvailable = false;
            }
        });
    }
    removeEtagWeakPrefix(etag) {
        return etag && etag.replace(util_1.WEAK_ETAG_REGEX, '');
    }
    initLocalServices() {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.log('Starting Parsoid & MCS');
            const runner = new service_runner_1.default();
            yield runner.start({
                num_workers: 0,
                services: [{
                        name: 'parsoid',
                        module: 'node_modules/parsoid/lib/index.js',
                        entrypoint: 'apiServiceWorker',
                        conf: {
                            timeouts: {
                                // request: 4 * 60 * 1000, // Default
                                request: 8 * 60 * 1000,
                            },
                            limits: {
                                wt2html: {
                                    // maxWikitextSize: 1000000, // Default
                                    maxWikitextSize: 1000000 * 4,
                                    // maxListItems: 30000, // Default
                                    maxListItems: 30000 * 4,
                                    // maxTableCells: 30000, // Default
                                    maxTableCells: 30000 * 4,
                                    // maxTransclusions: 10000, // Default
                                    maxTransclusions: 10000 * 4,
                                    // maxImages: 1000, // Default
                                    maxImages: 1000 * 4,
                                    // maxTokens: 1000000, // Default
                                    maxTokens: 1000000 * 4,
                                },
                            },
                            mwApis: [{
                                    uri: this.mw.apiUrl.href,
                                }],
                        },
                    }, {
                        name: 'mcs',
                        module: 'node_modules/service-mobileapp-node/app.js',
                        conf: {
                            port: 6927,
                            mwapi_req: {
                                method: 'post',
                                uri: this.mw.getApiQueryUrl().split('?')[0],
                                headers: {
                                    'user-agent': '{{user-agent}}',
                                },
                                body: '{{ default(request.query, {}) }}',
                            },
                            restbase_req: {
                                method: '{{request.method}}',
                                uri: 'http://localhost:8000/{{domain}}/v3/{+path}',
                                query: '{{ default(request.query, {}) }}',
                                headers: '{{request.headers}}',
                                body: '{{request.body}}',
                            },
                        },
                    }],
                logging: {
                    level: 'info',
                },
            });
        });
    }
    query(query) {
        return this.getJSON(this.mw.getApiQueryUrl(query));
    }
    getArticleDetailsIds(articleIds, shouldGetThumbnail = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let continuation;
            let finalProcessedResp;
            while (true) {
                const queryOpts = Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, this.getArticleQueryOpts(shouldGetThumbnail, true)), { titles: articleIds.join('|') }), (this.mwCapabilities.coordinatesAvailable ? { colimit: 'max' } : {})), (this.mw.getCategories ? {
                    cllimit: 'max',
                    clshow: '!hidden',
                } : {})), (continuation || {}));
                const queryString = util_1.objToQueryString(queryOpts);
                const reqUrl = this.mw.getApiQueryUrl(queryString);
                const resp = yield this.getJSON(reqUrl);
                Downloader.handleMWWarningsAndErrors(resp);
                let processedResponse = resp.query ? util_1.normalizeMwResponse(resp.query) : {};
                if (resp.continue) {
                    continuation = resp.continue;
                    const relevantDetails = this.stripNonContinuedProps(processedResponse);
                    finalProcessedResp = finalProcessedResp === undefined ? relevantDetails :
                        deepmerge_1.default(finalProcessedResp, relevantDetails);
                }
                else {
                    if (this.mw.getCategories) {
                        processedResponse = yield this.setArticleSubCategories(processedResponse);
                    }
                    finalProcessedResp = finalProcessedResp === undefined ? processedResponse
                        : deepmerge_1.default(finalProcessedResp, processedResponse);
                    break;
                }
            }
            return finalProcessedResp;
        });
    }
    getArticleDetailsNS(ns, gapcontinue = '') {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        return __awaiter(this, void 0, void 0, function* () {
            let queryContinuation;
            let finalProcessedResp;
            let gCont = null;
            while (true) {
                const queryOpts = Object.assign(Object.assign(Object.assign(Object.assign({}, this.getArticleQueryOpts()), (this.mwCapabilities.coordinatesAvailable ? { colimit: 'max' } : {})), (this.mw.getCategories ? {
                    cllimit: 'max',
                    clshow: '!hidden',
                } : {})), { rawcontinue: 'true', generator: 'allpages', gapfilterredir: 'nonredirects', gaplimit: 'max', gapnamespace: String(ns), gapcontinue });
                if (queryContinuation) {
                    queryOpts.cocontinue = (_b = (_a = queryContinuation === null || queryContinuation === void 0 ? void 0 : queryContinuation.coordinates) === null || _a === void 0 ? void 0 : _a.cocontinue) !== null && _b !== void 0 ? _b : queryOpts.cocontinue;
                    queryOpts.clcontinue = (_d = (_c = queryContinuation === null || queryContinuation === void 0 ? void 0 : queryContinuation.categories) === null || _c === void 0 ? void 0 : _c.clcontinue) !== null && _d !== void 0 ? _d : queryOpts.clcontinue;
                    queryOpts.picontinue = (_f = (_e = queryContinuation === null || queryContinuation === void 0 ? void 0 : queryContinuation.pageimages) === null || _e === void 0 ? void 0 : _e.picontinue) !== null && _f !== void 0 ? _f : queryOpts.picontinue;
                    queryOpts.rdcontinue = (_h = (_g = queryContinuation === null || queryContinuation === void 0 ? void 0 : queryContinuation.redirects) === null || _g === void 0 ? void 0 : _g.rdcontinue) !== null && _h !== void 0 ? _h : queryOpts.rdcontinue;
                }
                const queryString = util_1.objToQueryString(queryOpts);
                const reqUrl = this.mw.getApiQueryUrl(queryString);
                const resp = yield this.getJSON(reqUrl);
                Downloader.handleMWWarningsAndErrors(resp);
                let processedResponse = util_1.normalizeMwResponse(resp.query);
                gCont = (_l = (_k = (_j = resp['query-continue']) === null || _j === void 0 ? void 0 : _j.allpages) === null || _k === void 0 ? void 0 : _k.gapcontinue) !== null && _l !== void 0 ? _l : gCont;
                const queryComplete = Object.keys(resp['query-continue'] || {})
                    .filter((key) => key !== 'allpages')
                    .length === 0;
                if (!queryComplete) {
                    queryContinuation = resp['query-continue'];
                    const relevantDetails = this.stripNonContinuedProps(processedResponse);
                    finalProcessedResp = finalProcessedResp === undefined ? relevantDetails :
                        deepmerge_1.default(finalProcessedResp, relevantDetails);
                }
                else {
                    if (this.mw.getCategories) {
                        processedResponse = yield this.setArticleSubCategories(processedResponse);
                    }
                    finalProcessedResp = finalProcessedResp === undefined ? processedResponse
                        : deepmerge_1.default(finalProcessedResp, processedResponse);
                    break;
                }
            }
            return {
                articleDetails: finalProcessedResp,
                gapContinue: gCont,
            };
        });
    }
    getArticle(articleId, dump) {
        return __awaiter(this, void 0, void 0, function* () {
            const isMainPage = dump.isMainPage(articleId);
            const articleApiUrl = this.getArticleUrl(articleId, isMainPage);
            Logger_1.default.info(`Getting article [${articleId}] from ${articleApiUrl}`);
            const json = yield this.getJSON(articleApiUrl);
            if (json.error) {
                throw json.error;
            }
            return yield util_1.renderArticle(json, articleId, dump, this.mwCapabilities);
        });
    }
    getJSON(_url) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            const url = this.deserializeUrl(_url);
            yield self.claimRequest();
            return new Promise((resolve, reject) => {
                this.backoffCall(this.getJSONCb, url, (err, val) => {
                    self.releaseRequest();
                    if (err) {
                        const httpStatus = err.response && err.response.status;
                        Logger_1.default.warn(`Failed to get [${url}] [status=${httpStatus}]`);
                        reject(err);
                    }
                    else {
                        resolve(val);
                    }
                });
            });
        });
    }
    downloadContent(_url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!_url) {
                throw new Error(`Parameter [${_url}] is not a valid url`);
            }
            const url = this.deserializeUrl(_url);
            const self = this;
            yield self.claimRequest();
            return new Promise((resolve, reject) => {
                this.backoffCall(this.getContentCb, url, (err, val) => __awaiter(this, void 0, void 0, function* () {
                    self.releaseRequest();
                    if (err) {
                        const httpStatus = err.response && err.response.status;
                        Logger_1.default.warn(`Failed to get [${url}] [status=${httpStatus}]`);
                        reject(err);
                    }
                    else {
                        resolve(val);
                    }
                }));
            });
        });
    }
    canGetUrl(url) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield axios_1.default.get(url);
                return true;
            }
            catch (err) {
                return false;
            }
        });
    }
    getArticleUrl(articleId, isMainPage) {
        return `${isMainPage ? this.baseUrlForMainPage : this.baseUrl}${encodeURIComponent(articleId)}`;
    }
    stripNonContinuedProps(articleDetails, cont = {}) {
        const propsMap = {
            pageimages: ['thumbnail', 'pageimage'],
            coordinates: ['coordinates'],
            categories: ['categories'],
        };
        const keysToKeep = ['subCategories', 'revisions', 'redirects']
            .concat(Object.keys(cont).reduce((acc, key) => acc.concat(propsMap[key] || []), []));
        const items = Object.entries(articleDetails)
            .map(([aId, detail]) => {
            const newDetail = keysToKeep
                .reduce((acc, key) => {
                const val = detail[key];
                if (!val) {
                    return acc;
                }
                else {
                    return Object.assign(Object.assign({}, acc), { [key]: val });
                }
            }, {});
            return [
                aId,
                newDetail,
            ];
        });
        return items.reduce((acc, [key, detail]) => {
            return Object.assign(Object.assign({}, acc), { [key]: detail });
        }, {});
    }
    static handleMWWarningsAndErrors(resp) {
        var _a;
        if (resp.warnings)
            Logger_1.default.warn(`Got warning from MW Query ${JSON.stringify(resp.warnings, null, '\t')}`);
        if (((_a = resp.error) === null || _a === void 0 ? void 0 : _a.code) === util_1.DB_ERROR)
            throw new Error(`Got error from MW Query ${JSON.stringify(resp.error, null, '\t')}`);
        if (resp.error)
            Logger_1.default.log(`Got error from MW Query ${JSON.stringify(resp.warnings, null, '\t')}`);
    }
    getArticleQueryOpts(includePageimages = false, redirects = false) {
        const validNamespaceIds = this.mw.namespacesToMirror.map((ns) => this.mw.namespaces[ns].num);
        return {
            action: 'query',
            format: 'json',
            prop: `redirects|revisions${includePageimages ? '|pageimages' : ''}${this.mwCapabilities.coordinatesAvailable ? '|coordinates' : ''}${this.mw.getCategories ? '|categories' : ''}`,
            rdlimit: 'max',
            rdnamespace: validNamespaceIds.join('|'),
            redirects: (redirects ? true : undefined),
        };
    }
    setArticleSubCategories(articleDetails) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info(`Getting subCategories`);
            for (const [articleId, articleDetail] of Object.entries(articleDetails)) {
                const isCategoryArticle = articleDetail.ns === 14;
                if (isCategoryArticle) {
                    const categoryMembers = yield this.getSubCategories(articleId);
                    articleDetails[articleId].subCategories = categoryMembers.slice();
                }
            }
            return articleDetails;
        });
    }
    claimRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.activeRequests < this.maxActiveRequests) {
                this.activeRequests += 1;
                return null;
            }
            else {
                yield new Promise((resolve) => {
                    setTimeout(resolve, 200);
                });
                return this.claimRequest();
            }
        });
    }
    releaseRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            this.activeRequests -= 1;
            return null;
        });
    }
    getCompressedBody(resp) {
        return __awaiter(this, void 0, void 0, function* () {
            if (util_1.isBitmapImageMimeType(resp.headers['content-type'])) {
                if (util_1.isWebpCandidateImageMimeType(this.webp, resp.headers['content-type']) &&
                    !this.cssDependenceUrls.hasOwnProperty(resp.config.url)) {
                    resp.data = yield imagemin.buffer(resp.data, imageminOptions.get('webp').get(resp.headers['content-type']))
                        .catch((err) => __awaiter(this, void 0, void 0, function* () {
                        if (/Unsupported color conversion request/.test(err.stderr)) {
                            return yield imagemin.buffer(yield sharp_1.default(resp.data).toColorspace('srgb').toBuffer(), imageminOptions.get('webp').get(resp.headers['content-type']))
                                .catch(() => { return resp.data; })
                                .then((data) => {
                                resp.headers['content-type'] = 'image/webp';
                                return data;
                            });
                        }
                        else {
                            return yield imagemin.buffer(resp.data, imageminOptions.get('default').get(resp.headers['content-type']))
                                .catch(() => { return resp.data; });
                        }
                    }))
                        .then((data) => {
                        resp.headers['content-type'] = 'image/webp';
                        return data;
                    });
                    resp.headers.path_postfix = '.webp';
                }
                else {
                    resp.data = yield imagemin.buffer(resp.data, imageminOptions.get('default').get(resp.headers['content-type']))
                        .catch(() => { return resp.data; });
                }
                return true;
            }
            return false;
        });
    }
    downloadImage(url, handler) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.s3.downloadBlob(util_1.stripHttpFromUrl(url), this.webp ? 'webp' : '1').then((s3Resp) => __awaiter(this, void 0, void 0, function* () {
                    var _a;
                    if ((_a = s3Resp === null || s3Resp === void 0 ? void 0 : s3Resp.Metadata) === null || _a === void 0 ? void 0 : _a.etag) {
                        this.arrayBufferRequestOptions.headers['If-None-Match']
                            = this.removeEtagWeakPrefix(s3Resp.Metadata.etag);
                    }
                    const mwResp = yield axios_1.default(url, this.arrayBufferRequestOptions);
                    // Most of the images after uploading once will always have
                    // 304 status, until modified. We need to have
                    // isWebpCandidateImageUrl() as fallback check because
                    // of https://phabricator.wikimedia.org/T265006
                    if (mwResp.status === 304) {
                        const headers = ((_a) => {
                            var { Body } = _a, o = __rest(_a, ["Body"]);
                            return o;
                        })(s3Resp);
                        if (mwResp.headers['content-type'] ?
                            util_1.isWebpCandidateImageMimeType(this.webp, mwResp.headers['content-type']) :
                            this.webp && util_1.isWebpCandidateImageUrl(mwResp.config.url) &&
                                !this.cssDependenceUrls.hasOwnProperty(mwResp.config.url)) {
                            headers.path_postfix = '.webp';
                            headers['content-type'] = 'image/webp';
                        }
                        handler(null, {
                            responseHeaders: headers,
                            content: s3Resp.Body,
                        });
                        return;
                    }
                    // Compress content
                    yield this.getCompressedBody(mwResp);
                    // Check for the etag and upload
                    const etag = this.removeEtagWeakPrefix(mwResp.headers.etag);
                    if (etag) {
                        this.s3.uploadBlob(util_1.stripHttpFromUrl(url), mwResp.data, etag, this.webp ? 'webp' : '1');
                    }
                    handler(null, {
                        responseHeaders: mwResp.headers,
                        content: mwResp.data,
                    });
                })).catch((err) => {
                    this.errHandler(err, url, handler);
                });
            }
            catch (err) {
                this.errHandler(err, url, handler);
            }
        });
    }
    errHandler(err, url, handler) {
        if (err.response && err.response.status === 429) {
            Logger_1.default.log(`Received a [status=429], slowing down`);
            const newMaxActiveRequests = Math.max(Math.ceil(this.maxActiveRequests * 0.9), 1);
            Logger_1.default.log(`Setting maxActiveRequests from [${this.maxActiveRequests}] to [${newMaxActiveRequests}]`);
            this.maxActiveRequests = newMaxActiveRequests;
        }
        Logger_1.default.log(`Not able to download content for ${url} due to ${err}`);
        handler(err);
    }
    getSubCategories(articleId, continueStr = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const { query, continue: cont } = yield this.getJSON(this.mw.subCategoriesApiUrl(articleId, continueStr));
            const items = query.categorymembers.filter((a) => a && a.title);
            if (cont && cont.cmcontinue) {
                const nextItems = yield this.getSubCategories(articleId, cont.cmcontinue);
                return items.concat(nextItems);
            }
            else {
                return items;
            }
        });
    }
    backoffCall(handler, url, callback) {
        const call = backoff.call(handler, url, callback);
        call.setStrategy(this.backoffOptions.strategy);
        call.retryIf(this.backoffOptions.retryIf);
        call.failAfter(this.backoffOptions.failAfter);
        call.on('backoff', this.backoffOptions.backoffHandler);
        call.start();
    }
}
exports.default = Downloader;
//# sourceMappingURL=Downloader.js.map