"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dump = void 0;
const pathParser = __importStar(require("path"));
const urlParser = __importStar(require("url"));
const fs_1 = require("fs");
const domino = __importStar(require("domino"));
const Logger_1 = __importDefault(require("./Logger"));
const util_1 = require("./util");
class Dump {
    constructor(format, opts, mwMetaData, customProcessor) {
        this.isMainPage = (articleId) => { return this.mwMetaData.mainPage === articleId ? true : false; };
        this.status = {
            files: {
                success: 0,
                fail: 0,
            },
            articles: {
                success: 0,
                fail: 0,
            },
            redirects: {
                written: 0,
            },
        };
        this.mwMetaData = mwMetaData;
        this.opts = opts;
        this.customProcessor = customProcessor;
        const [formatStr, formatFlavour] = format.split(':');
        this.nopic = formatStr.includes('nopic');
        this.novid = formatStr.includes('novid');
        this.nopdf = formatStr.includes('nopdf');
        this.nodet = formatStr.includes('nodet');
        this.formatFlavour = formatFlavour;
        /* Get language specific strings */
        this.strings = util_1.getStringsForLang(mwMetaData.langIso2 || 'en', 'en');
    }
    computeFlavour() {
        const flavour = [];
        if (typeof this.formatFlavour === 'string') {
            flavour.push(this.formatFlavour);
        }
        else {
            if (this.nopic) {
                flavour.push('nopic');
            }
            else if (this.nopdf) {
                flavour.push('nopdf');
            }
            else if (this.novid && !this.nodet) {
                flavour.push('novid');
            }
            if (this.nodet) {
                flavour.push('nodet');
            }
        }
        return flavour.join('_');
    }
    computeFilenameRadical(withoutSelection, withoutFlavour, withoutDate) {
        let radical;
        if (this.opts.filenamePrefix) {
            radical = this.opts.filenamePrefix;
        }
        else {
            radical = `${this.mwMetaData.creator.charAt(0).toLowerCase() + this.mwMetaData.creator.substr(1)}_`;
            const hostParts = urlParser.parse(this.mwMetaData.webUrl).hostname.split('.');
            let langSuffix = this.mwMetaData.langIso2;
            for (const part of hostParts) {
                if (part === this.mwMetaData.langIso3) {
                    langSuffix = part;
                    break;
                }
            }
            radical += langSuffix;
        }
        if (!withoutSelection && !this.opts.filenamePrefix) {
            if (this.opts.articleList) {
                radical += `_${pathParser.basename(this.opts.articleList).toLowerCase().replace(/\.\w{3}$/, '')}`;
            }
            else {
                radical += '_all';
            }
        }
        if (!withoutFlavour && this.computeFlavour()) {
            radical += `_${this.computeFlavour()}`;
        }
        if (!withoutDate) {
            radical += `_${this.opts.filenameDate}`;
        }
        return radical;
    }
    checkResume() {
        if (this.opts.resume) {
            const zimPath = this.computeZimRootPath();
            if (fs_1.existsSync(zimPath)) {
                Logger_1.default.log(`${zimPath} is already done, skip dumping & ZIM file generation`);
                throw new Error(`TODO: IMPLEMENT RESUME`);
            }
        }
    }
    computeZimTags() {
        /* Add tag and avoid duplicates */
        function addTagWithoutDuplicate(newTag) {
            if (!tags.find((tag) => tag === newTag)) {
                tags.push(newTag);
            }
        }
        /* Split tags in a list */
        let tags = (this.opts.tags || '').split(';');
        /* Add Mediawiki hostname radical as a tag */
        const mwUrlHostParts = urlParser.parse(this.mwMetaData.baseUrl).host.split('.');
        const mwUrlHostPartsRadical = mwUrlHostParts.length > 1
            ? mwUrlHostParts[mwUrlHostParts.length - 2]
            : mwUrlHostParts[mwUrlHostParts.length - 1];
        const mwUrlHostPartsTag = mwUrlHostPartsRadical.toLowerCase();
        addTagWithoutDuplicate(mwUrlHostPartsTag);
        /* Famous Web sites have their own hidden category */
        if (mwUrlHostPartsTag.match(/^(gutenberg|phet|psiram|stack_exchange|ted|vikidia|wikibooks|wikinews|wikipedia|wikiquote|wikisource|wikiversity|wikivoyage|wiktionary)$/)) {
            addTagWithoutDuplicate('_category:' + mwUrlHostPartsTag);
        }
        /* Add --format tags */
        if (this.nopic) {
            addTagWithoutDuplicate('_pictures:no');
            addTagWithoutDuplicate('_videos:no');
        }
        else if (this.novid) {
            addTagWithoutDuplicate('_pictures:yes');
            addTagWithoutDuplicate('_videos:no');
        }
        if (this.nodet) {
            addTagWithoutDuplicate('_videos:no');
            addTagWithoutDuplicate('_details:no');
        }
        else {
            addTagWithoutDuplicate('_details:yes');
        }
        /* Add proper _ftindex tag */
        addTagWithoutDuplicate('_ftindex:' + (this.opts.withoutZimFullTextIndex ? 'no' : 'yes'));
        /* Remove empty tags */
        tags = tags.filter((x) => x);
        return tags.join(';');
    }
    computeZimRootPath() {
        let zimRootPath = this.opts.outputDirectory[0] === '/' ? this.opts.outputDirectory : `${pathParser.resolve(process.cwd(), this.opts.outputDirectory)}/`;
        zimRootPath += `${this.computeFilenameRadical()}.zim`;
        return zimRootPath;
    }
    getRelevantStylesheetUrls(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            const sheetUrls = [];
            /* Load main page to see which CSS files are needed */
            const { content } = yield downloader.downloadContent(this.mwMetaData.webUrl);
            const html = content.toString();
            const doc = domino.createDocument(html);
            const links = Array.from(doc.getElementsByTagName('link'));
            /* Go through all CSS links */
            for (const link of links) {
                if (link.getAttribute('rel') === 'stylesheet' &&
                    link.getAttribute('href') &&
                    !link.getAttribute('href').match('^data')) {
                    sheetUrls.push(link);
                }
            }
            /* Push Mediawiki:Offline.css (at the end) */
            // TODO: Weak URL (might fail in a number of cases where the wiki path is not like on Wikipedia)
            const offlineCssUrl = downloader.mw.getWebArticleUrlRaw('Mediawiki:offline.css');
            if (yield downloader.canGetUrl(offlineCssUrl)) {
                sheetUrls.push(offlineCssUrl);
            }
            return sheetUrls.filter((a) => a);
        });
    }
}
exports.Dump = Dump;
//# sourceMappingURL=Dump.js.map