"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = __importDefault(require("url"));
const pathParser = __importStar(require("path"));
const Logger_1 = __importDefault(require("./Logger"));
const util = __importStar(require("./util"));
const domino = __importStar(require("domino"));
const util_1 = require("./util");
const axios_1 = __importDefault(require("axios"));
const querystring_1 = __importDefault(require("querystring"));
const semver_1 = __importDefault(require("semver"));
class MediaWiki {
    constructor(config) {
        var _a, _b, _c, _d, _e, _f;
        this.namespaces = {};
        this.namespacesToMirror = [];
        this.domain = config.domain || '';
        this.username = config.username;
        this.password = config.password;
        this.getCategories = config.getCategories;
        this.baseUrl = new URL(util_1.ensureTrailingChar(config.base, '/'));
        this.apiPath = (_a = config.apiPath) !== null && _a !== void 0 ? _a : 'w/api.php';
        this.wikiPath = (_b = config.wikiPath) !== null && _b !== void 0 ? _b : util_1.DEFAULT_WIKI_PATH;
        this.webUrl = new URL(this.wikiPath, this.baseUrl);
        this.apiUrl = new URL(`${this.apiPath}?`, this.baseUrl);
        this.veApiUrl = new URL(`${this.apiUrl.href}action=visualeditor&mobileformat=html&format=json&paction=parse&page=`);
        this.restApiUrl = new URL(util_1.ensureTrailingChar(new URL((_c = config.restApiPath) !== null && _c !== void 0 ? _c : 'api/rest_v1', this.baseUrl.href).toString(), '/'));
        this.mobileRestApiUrl = new URL(util_1.ensureTrailingChar(new URL((_d = config.restApiPath) !== null && _d !== void 0 ? _d : 'api/rest_v1/page/mobile-sections', this.baseUrl.href).toString(), '/'));
        this.desktopRestApiUrl = new URL(util_1.ensureTrailingChar(new URL((_e = config.restApiPath) !== null && _e !== void 0 ? _e : 'api/rest_v1/page/html', this.baseUrl.href).toString(), '/'));
        this.modulePath = `${url_1.default.resolve(this.baseUrl.href, (_f = config.modulePath) !== null && _f !== void 0 ? _f : 'w/load.php')}?`;
        this.articleApiUrlBase = `${this.apiUrl.href}action=parse&format=json&prop=${encodeURI('modules|jsconfigvars|headhtml')}&page=`;
    }
    login(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.username && this.password) {
                let url = this.apiUrl.href;
                // Add domain if configured
                if (this.domain) {
                    url = `${url}lgdomain=${this.domain}&`;
                }
                // Getting token to login.
                const { content, responseHeaders } = yield downloader.downloadContent(url + 'action=query&meta=tokens&type=login&format=json');
                // Logging in
                yield axios_1.default(this.apiUrl.href, {
                    data: querystring_1.default.stringify({
                        action: 'login',
                        format: 'json',
                        lgname: this.username,
                        lgpassword: this.password,
                        lgtoken: JSON.parse(content.toString()).query.tokens.logintoken,
                    }),
                    headers: {
                        Cookie: responseHeaders['set-cookie'].join(';'),
                        'Content-Type': 'application/x-www-form-urlencoded'
                    },
                    method: 'POST',
                })
                    .then((resp) => {
                    if (resp.data.login.result !== 'Success') {
                        throw new Error('Login Failed');
                    }
                    downloader.loginCookie = resp.headers['set-cookie'].join(';');
                    downloader.arrayBufferRequestOptions.headers.cookie = downloader.loginCookie;
                    downloader.jsonRequestOptions.headers.cookie = downloader.loginCookie;
                    downloader.streamRequestOptions.headers.cookie = downloader.loginCookie;
                })
                    .catch((err) => {
                    throw err;
                });
            }
        });
    }
    // In all the url methods below:
    // * encodeURIComponent is mandatory for languages with illegal letters for uri (fa.wikipedia.org)
    // * encodeURI is mandatory to encode the pipes '|' but the '&' and '=' must not be encoded
    siteInfoUrl() {
        return `${this.apiUrl.href}action=query&meta=siteinfo&format=json`;
    }
    articleApiUrl(articleId) {
        return `${this.articleApiUrlBase}${encodeURIComponent(articleId)}`;
    }
    subCategoriesApiUrl(articleId, continueStr = '') {
        return `${this.apiUrl.href}action=query&list=categorymembers&cmtype=subcat&cmlimit=max&format=json&cmtitle=${encodeURIComponent(articleId)}&cmcontinue=${continueStr}`;
    }
    getVeApiArticleUrl(articleId) {
        return `${this.veApiUrl.href}${encodeURIComponent(articleId)}`;
    }
    getDesktopRestApiArticleUrl(articleId) {
        return `${this.desktopRestApiUrl.href}${encodeURIComponent(articleId)}`;
    }
    getMobileRestApiArticleUrl(articleId) {
        return `${this.mobileRestApiUrl.href}${encodeURIComponent(articleId)}`;
    }
    getApiQueryUrl(query = '') {
        return `${this.apiUrl.href}${query}`;
    }
    getWebArticleUrlRaw(articleId) {
        return `${this.webUrl.href}?title=${encodeURIComponent(articleId)}&action=raw`;
    }
    getNamespaces(addNamespaces, downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            const url = `${this.apiUrl.href}action=query&meta=siteinfo&siprop=namespaces|namespacealiases&format=json`;
            const json = yield downloader.getJSON(url);
            ['namespaces', 'namespacealiases'].forEach((type) => {
                const entries = json.query[type];
                Object.keys(entries).forEach((key) => {
                    const entry = entries[key];
                    const name = entry['*'];
                    const num = entry.id;
                    const allowedSubpages = ('subpages' in entry);
                    const isContent = !!(entry.content !== undefined || util.contains(addNamespaces, num));
                    const canonical = entry.canonical ? entry.canonical : '';
                    const details = { num, allowedSubpages, isContent };
                    /* Namespaces in local language */
                    self.namespaces[util.lcFirst(name)] = details;
                    self.namespaces[util.ucFirst(name)] = details;
                    /* Namespaces in English (if available) */
                    if (canonical) {
                        self.namespaces[util.lcFirst(canonical)] = details;
                        self.namespaces[util.ucFirst(canonical)] = details;
                    }
                    /* Is content to mirror */
                    if (isContent) {
                        self.namespacesToMirror.push(name);
                    }
                });
            });
        });
    }
    extractPageTitleFromHref(href) {
        try {
            const pathname = new URL(href, this.baseUrl).pathname;
            // Local relative URL
            if (href.indexOf('./') === 0) {
                return util.decodeURIComponent(pathname.substr(1));
            }
            // Absolute path
            if (pathname.indexOf(this.webUrl.pathname) === 0) {
                return util.decodeURIComponent(pathname.substr(this.webUrl.pathname.length));
            }
            // If local Parsoid, links start with './wiki/' (and no easy way
            // to identify we are using a local parsoid so far because of bad
            // architecture).
            if (pathname.indexOf('/wiki/') === 0) {
                return util.decodeURIComponent(pathname.substr(6));
            }
            const isPaginatedRegExp = /\/[0-9]+(\.|$)/;
            const isPaginated = isPaginatedRegExp.test(href);
            if (isPaginated) {
                const withoutDotHtml = href.split('.').slice(0, -1).join('.');
                const lastTwoSlashes = withoutDotHtml.split('/').slice(-2).join('/');
                return lastTwoSlashes;
            }
            if (pathParser.parse(href).dir.includes('../')) {
                return pathParser.parse(href).name;
            }
            return null; /* Interwiki link? -- return null */
        }
        catch (error) {
            Logger_1.default.warn(`Unable to parse href ${href}`);
            return null;
        }
    }
    getCreatorName() {
        /*
         * Find a suitable name to use for ZIM (content) creator
         * Heuristic: Use basename of the domain unless
         * - it happens to be a wikimedia project OR
         * - some domain where the second part of the hostname is longer than the first part
         */
        const hostParts = this.baseUrl.hostname.split('.');
        let creator = hostParts[0];
        if (hostParts.length > 1) {
            const wmProjects = new Set([
                'wikipedia',
                'wikisource',
                'wikibooks',
                'wikiquote',
                'wikivoyage',
                'wikiversity',
                'wikinews',
                'wiktionary',
            ]);
            if (wmProjects.has(hostParts[1]) || hostParts[0].length < hostParts[1].length) {
                creator = hostParts[1]; // Name of the wikimedia project
            }
        }
        creator = creator.charAt(0).toUpperCase() + creator.substr(1);
        return creator;
    }
    getTextDirection(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.log('Getting text direction...');
            const { content } = yield downloader.downloadContent(this.webUrl.href);
            const body = content.toString();
            const doc = domino.createDocument(body);
            const contentNode = doc.getElementById('mw-content-text');
            const languageDirectionRegex = /"pageLanguageDir":"(.*?)"/;
            const parts = languageDirectionRegex.exec(body);
            let isLtr = true;
            if (parts && parts[1]) {
                isLtr = (parts[1] === 'ltr');
            }
            else if (contentNode) {
                isLtr = (contentNode.getAttribute('dir') === 'ltr');
            }
            else {
                Logger_1.default.log('Unable to get the language direction, fallback to ltr');
                isLtr = true;
            }
            const textDir = isLtr ? 'ltr' : 'rtl';
            Logger_1.default.log(`Text direction is [${textDir}]`);
            return textDir;
        });
    }
    getSiteInfo(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            Logger_1.default.log('Getting site info...');
            const query = `action=query&meta=siteinfo&format=json&siprop=general|namespaces|statistics|variables|category|wikidesc`;
            const body = yield downloader.query(query);
            const entries = body.query.general;
            // Checking mediawiki version
            const mwVersion = semver_1.default.coerce(entries.generator).raw;
            const mwMinimalVersion = 1.27;
            if (!entries.generator || !semver_1.default.satisfies(mwVersion, `>=${mwMinimalVersion}`)) {
                throw new Error(`Mediawiki version ${mwVersion} not supported should be >=${mwMinimalVersion}`);
            }
            // Base will contain the default encoded article id for the wiki.
            const mainPage = decodeURIComponent(entries.base.split('/').pop());
            const siteName = entries.sitename;
            const langs = [entries.lang].concat(entries.fallback.map((e) => e.code));
            const [langIso2, langIso3] = yield Promise.all(langs.map((lang) => __awaiter(this, void 0, void 0, function* () {
                let langIso3;
                try {
                    langIso3 = yield util.getIso3(lang);
                }
                catch (err) {
                    langIso3 = lang;
                }
                try {
                    return [
                        lang,
                        langIso3,
                    ];
                }
                catch (err) {
                    return false;
                }
            }))).then((possibleLangPairs) => {
                possibleLangPairs = possibleLangPairs.filter((a) => a);
                return possibleLangPairs[0] || ['en', 'eng'];
            });
            return {
                mainPage,
                siteName,
                langIso2,
                langIso3,
            };
        });
    }
    getSubTitle(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.log('Getting sub-title...');
            const { content } = yield downloader.downloadContent(this.webUrl.href);
            const html = content.toString();
            const doc = domino.createDocument(html);
            const subTitleNode = doc.getElementById('siteSub');
            return subTitleNode ? subTitleNode.innerHTML : '';
        });
    }
    getMwMetaData(downloader) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.metaData) {
                return this.metaData;
            }
            const creator = this.getCreatorName() || 'Kiwix';
            const [textDir, { langIso2, langIso3, mainPage, siteName }, subTitle,] = yield Promise.all([
                this.getTextDirection(downloader),
                this.getSiteInfo(downloader),
                this.getSubTitle(downloader),
            ]);
            const mwMetaData = {
                webUrl: this.webUrl.href,
                apiUrl: this.apiUrl.href,
                modulePath: this.modulePath,
                webUrlPath: this.webUrl.pathname,
                wikiPath: this.wikiPath,
                baseUrl: this.baseUrl.href,
                apiPath: this.apiPath,
                domain: this.domain,
                textDir: textDir,
                langIso2,
                langIso3,
                title: siteName,
                subTitle,
                creator,
                mainPage,
            };
            this.metaData = mwMetaData;
            return mwMetaData;
        });
    }
}
exports.default = MediaWiki;
//# sourceMappingURL=MediaWiki.js.map