#!/bin/sh
':' //# -*- mode: js -*-; exec /usr/bin/env node --max-old-space-size=9000 --stack-size=42000 "$0" "$@"
'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs"));
const parameterList_1 = require("./parameterList");
const mwofflinerLib = __importStar(require("./mwoffliner.lib"));
/************************************/
/* Command Parsing ******************/
/************************************/
const argv = yargs_1.default
    .help('help')
    .usage(`Create a fancy HTML dump of a Mediawiki instance in a ZIM file
  Usage: $0
  Example, as a system tool:
  mwoffliner --mwUrl=https://en.wikipedia.org/ --adminEmail=foo@bar.net
  Or, as a node script:
  node mwoffliner.js --mwUrl=https://en.wikipedia.org/ --adminEmail=foo@bar.net
  Or, as a npm script: '
  npm run mwoffliner -- --mwUrl=https://en.wikipedia.org/ --adminEmail=foo@bar.net`)
    .describe(parameterList_1.parameterDescriptions)
    .require(parameterList_1.requiredParams)
    .strict().argv;
/* ***********************************/
/* TMPDIR OVERRIDE HAS TO BE HANDLED */
/* AT THE REALLY BEGIN               */
/* ***********************************/
const fs_1 = __importDefault(require("fs"));
if (argv.osTmpDir) {
    const osTmpDir = argv.osTmpDir;
    try {
        if (fs_1.default.statSync(osTmpDir)) {
            process.env.TMPDIR = osTmpDir;
        }
        else {
            throw new Error();
        }
    }
    catch (_a) {
        console.error(`--osTmpDir value [${osTmpDir}] is not valid`);
        process.exit(2);
    }
}
/* ***********************/
/* TESTING ALL ARGUMENTS */
/* ***********************/
const sanitize_argument_1 = require("./sanitize-argument");
const execStartTime = Date.now();
sanitize_argument_1.sanitize_all(argv)
    .then(() => {
    /* ***********************************/
    /* GO THROUGH ENTRY POINT            */
    /* ***********************************/
    mwofflinerLib
        .execute(argv)
        .then(() => {
        console.info(`Finished running mwoffliner after [${Math.round((Date.now() - execStartTime) / 1000)}s]`);
        process.exit(0);
    })
        .catch((err) => {
        errorHandler(err);
    });
})
    .catch((err) => {
    errorHandler(err);
});
// Hack to allow serializing of Errors
// https://stackoverflow.com/questions/18391212/is-it-not-possible-to-stringify-an-error-using-json-stringify
if (!('toJSON' in Error.prototype)) {
    Object.defineProperty(Error.prototype, 'toJSON', {
        value() {
            const alt = {};
            Object.getOwnPropertyNames(this).forEach(function (key) {
                alt[key] = this[key];
            }, this);
            return alt;
        },
        configurable: true,
        writable: true,
    });
}
function errorHandler(err) {
    let loggableErr = err;
    try {
        loggableErr = JSON.stringify(err, null, '\t');
    }
    catch (err) {
        /* NOOP */
    }
    console.error(`Failed to run mwoffliner after [${Math.round((Date.now() - execStartTime) / 1000)}s]:`, loggableErr);
    if (err && err.message) {
        console.error(`\n\n**********\n\n${err.message}\n\n**********\n\n`);
    }
    process.exit(2);
}
//# sourceMappingURL=cli.js.map