"use strict";
// tslint:disable-next-line: no-reference
///<reference path="./types.d.ts" />
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.execute = void 0;
/* ********************************** */
/* MODULE VARIABLE SECTION ********** */
/* ********************************** */
const fs_1 = __importStar(require("fs"));
const os_1 = __importDefault(require("os"));
const p_map_1 = __importDefault(require("p-map"));
const axios_1 = __importDefault(require("axios"));
const sharp_1 = __importDefault(require("sharp"));
const domino_1 = __importDefault(require("domino"));
const rimraf_1 = __importDefault(require("rimraf"));
const url_1 = __importDefault(require("url"));
const semver_1 = __importDefault(require("semver"));
const path = __importStar(require("path"));
const QueryStringParser = __importStar(require("querystring"));
const libzim_1 = require("@openzim/libzim");
const stores_1 = require("./stores");
const util_1 = require("./util");
const S3_1 = __importDefault(require("./S3"));
const Redis_1 = __importDefault(require("./Redis"));
const Logger_1 = __importDefault(require("./Logger"));
const Dump_1 = require("./Dump");
const config_1 = require("./config");
const MediaWiki_1 = __importDefault(require("./MediaWiki"));
const Downloader_1 = __importDefault(require("./Downloader"));
const redirects_1 = require("./util/redirects");
const Templates_1 = require("./Templates");
const saveArticles_1 = require("./util/saveArticles");
const categories_1 = require("./util/categories");
const packageJSON = JSON.parse(fs_1.readFileSync(path.join(__dirname, '../package.json'), 'utf8'));
function closeRedis(redis) {
    Logger_1.default.log(`Flushing Redis DBs`);
    if (redis.client.connected) {
        stores_1.filesToDownloadXPath.flush();
        stores_1.filesToRetryXPath.flush();
        stores_1.articleDetailXId.flush();
        stores_1.redirectsXId.flush();
        redis.client.quit();
    }
}
function execute(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        /* ********************************* */
        /* CUSTOM VARIABLE SECTION ********* */
        /* ********************************* */
        const { speed: _speed, adminEmail, verbose, minifyHtml, keepEmptyParagraphs, mwUrl, mwWikiPath, mwApiPath, mwRestApiPath, mwModulePath, mwDomain, mwUsername, mwPassword, requestTimeout, customMainPage, customZimTitle, customZimDescription, customZimTags, withoutZimFullTextIndex, webp, format, filenamePrefix, resume, publisher: _publisher, outputDirectory: _outputDirectory, addNamespaces: _addNamespaces, articleList: _articleList, customZimFavicon: _customZimFavicon, optimisationCacheUrl, noLocalParserFallback, forceLocalParser, customFlavour, zstd, } = argv;
        process.verbose = !!verbose;
        Logger_1.default.log(`Starting mwoffliner v${packageJSON.version}...`);
        let articleList = _articleList ? String(_articleList) : _articleList;
        const publisher = _publisher || config_1.config.defaults.publisher;
        let customZimFavicon = _customZimFavicon;
        /* HTTP user-agent string */
        // const adminEmail = argv.adminEmail;
        if (!util_1.isValidEmail(adminEmail)) {
            throw new Error(`Admin email [${adminEmail}] is not valid`);
        }
        /* Number of parallel requests */
        if (_speed && isNaN(_speed)) {
            throw new Error('speed is not a number, please give a number value to --speed');
        }
        const cpuCount = os_1.default.cpus().length;
        const speed = Math.max(1, Math.round(cpuCount * (_speed || 1) * 2));
        /* Check Node.js version */
        const nodeVersionSatisfiesPackage = semver_1.default.satisfies(process.version, packageJSON.engines.node);
        if (!nodeVersionSatisfiesPackage) {
            Logger_1.default.warn(`***********\n\n\tCurrent node version is [${process.version}]. We recommend [${packageJSON.engines.node}]\n\n***********`);
        }
        /* Instanciate custom flavour module */
        Logger_1.default.info(`Using custom flavour: ${customFlavour || 'no'}`);
        const customProcessor = customFlavour ?
            new (require(customFlavour)) : null;
        // Check for S3 creds
        if (optimisationCacheUrl) {
            // Decompose the url with path and other S3 creds
            const s3UrlObj = url_1.default.parse(optimisationCacheUrl);
            const queryReader = QueryStringParser.parse(s3UrlObj.query);
            const s3Url = (s3UrlObj.protocol || 'https:') + '//' + (s3UrlObj.host || '') + (s3UrlObj.pathname || '');
            this.s3Obj = new S3_1.default(s3Url, queryReader);
            yield this.s3Obj.initialise().then(() => {
                Logger_1.default.log('Successfully logged in S3');
            });
        }
        // Extract S3 obj to pass to downloader class
        const s3 = this.s3Obj ? this.s3Obj : {};
        /* Wikipedia/... URL; Normalize by adding trailing / as necessary */
        const mw = new MediaWiki_1.default({
            getCategories: !!argv.getCategories,
            apiPath: mwApiPath,
            restApiPath: mwRestApiPath,
            modulePath: mwModulePath,
            base: mwUrl,
            domain: mwDomain,
            password: mwPassword,
            username: mwUsername,
            wikiPath: mwWikiPath,
        });
        /* Download helpers; TODO: Merge with something else / expand this. */
        const downloader = new Downloader_1.default({
            mw,
            uaString: `${config_1.config.userAgent} (${adminEmail})`,
            speed,
            reqTimeout: requestTimeout * 1000 || config_1.config.defaults.requestTimeout,
            noLocalParserFallback,
            forceLocalParser,
            optimisationCacheUrl,
            s3,
            webp,
        });
        /* perform login */
        yield mw.login(downloader);
        /* Get MediaWiki Info */
        let mwMetaData;
        try {
            mwMetaData = yield mw.getMwMetaData(downloader);
        }
        catch (err) {
            Logger_1.default.error(`FATAL - Failed to get MediaWiki Metadata`);
            throw err;
        }
        // Sanitizing main page
        let mainPage = articleList ? '' : mwMetaData.mainPage;
        if (customMainPage) {
            mainPage = customMainPage;
            const mainPageUrl = mw.webUrl + encodeURIComponent(mainPage);
            if (!(yield downloader.checkApiAvailabilty(mainPageUrl))) {
                throw new Error(`customMainPage doesn't return 200 status code for url ${mainPageUrl}`);
            }
        }
        yield downloader.checkCapabilities(mwMetaData.mainPage);
        yield downloader.setBaseUrls();
        const redis = new Redis_1.default(argv, config_1.config);
        stores_1.populateArticleDetail(redis.client);
        stores_1.populateRedirects(redis.client);
        stores_1.populateFilesToDownload(redis.client);
        stores_1.populateFilesToRetry(redis.client);
        // Output directory
        const outputDirectory = path.isAbsolute(_outputDirectory || '') ?
            _outputDirectory : path.join(process.cwd(), _outputDirectory || 'out');
        yield util_1.mkdirPromise(outputDirectory);
        Logger_1.default.log(`Using output directory ${outputDirectory}`);
        // Temporary directory
        const tmpDirectory = path.resolve(os_1.default.tmpdir(), `mwoffliner-${Date.now()}`);
        try {
            Logger_1.default.info(`Creating temporary directory [${tmpDirectory}]`);
            yield util_1.mkdirPromise(tmpDirectory);
        }
        catch (err) {
            Logger_1.default.error(`Failed to create temporary directory, exiting`, err);
            throw err;
        }
        Logger_1.default.log(`Using temporary directory ${tmpDirectory}`);
        process.on('exit', (code) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.log(`Exiting with code [${code}]`);
            Logger_1.default.log(`Deleting temporary directory [${tmpDirectory}]`);
            rimraf_1.default.sync(tmpDirectory);
        }));
        process.on('SIGTERM', () => {
            Logger_1.default.log(`SIGTERM`);
            closeRedis(redis);
            process.exit(128 + 15);
        });
        process.on('SIGINT', () => {
            Logger_1.default.log(`SIGINT`);
            closeRedis(redis);
            process.exit(128 + 2);
        });
        /* *********************************** */
        /*       SYSTEM VARIABLE SECTION       */
        /* *********************************** */
        const dumpFormats = util_1.getDumps(format);
        const addNamespaces = _addNamespaces ? String(_addNamespaces).split(',').map((a) => Number(a)) : [];
        /* ZIM custom Favicon */
        if (customZimFavicon) {
            const faviconPath = path.join(tmpDirectory, 'favicon.png');
            const faviconIsRemote = customZimFavicon.includes('http');
            Logger_1.default.log(`${faviconIsRemote ? 'Downloading' : 'Moving'} custom favicon to [${faviconPath}]`);
            let content;
            if (faviconIsRemote) {
                Logger_1.default.log(`Downloading remote zim favicon from [${customZimFavicon}]`);
                content = yield axios_1.default.get(customZimFavicon, downloader.arrayBufferRequestOptions)
                    .then((a) => a.data)
                    .catch((err) => {
                    throw new Error(`Failed to download custom zim favicon from [${customZimFavicon}]`);
                });
            }
            else {
                try {
                    content = fs_1.default.readFileSync(customZimFavicon);
                }
                catch (err) {
                    throw new Error(`Failed to read custom zim favicon from [${customZimFavicon}]`);
                }
            }
            fs_1.default.writeFileSync(faviconPath, content);
            customZimFavicon = faviconPath;
            if (!fs_1.default.existsSync(customZimFavicon)) {
                throw new Error(`Path ${customZimFavicon} is not a valid PNG file.`);
            }
        }
        /* ********************************* */
        /* GET CONTENT ********************* */
        /* ********************************* */
        if (articleList && articleList.includes('http')) {
            try {
                const fileName = articleList.split('/').slice(-1)[0];
                const tmpArticleListPath = path.join(tmpDirectory, fileName);
                Logger_1.default.log(`Downloading article list from [${articleList}] to [${tmpArticleListPath}]`);
                const { data: articleListContentStream } = yield axios_1.default.get(articleList, downloader.streamRequestOptions);
                const articleListWriteStream = fs_1.default.createWriteStream(tmpArticleListPath);
                yield new Promise((resolve, reject) => {
                    articleListContentStream
                        .pipe(articleListWriteStream)
                        .on('error', (err) => reject(err))
                        .on('close', resolve);
                });
                articleList = tmpArticleListPath;
            }
            catch (err) {
                throw new Error(`Failed to download article list from [${articleList}]`);
            }
        }
        let articleListLines;
        try {
            articleListLines = articleList ? fs_1.default.readFileSync(articleList).toString().split('\n').
                map(a => a.replace(/\r/gm, '')).filter((a) => a) : [];
            Logger_1.default.info(`ArticleList has [${articleListLines.length}] items`);
        }
        catch (err) {
            Logger_1.default.error(`Failed to read articleList from [${articleList}]`, err);
            throw err;
        }
        yield mw.getNamespaces(addNamespaces, downloader);
        Logger_1.default.info(`Getting article ids`);
        yield redirects_1.getArticleIds(downloader, mw, mainPage, articleList ? articleListLines : null);
        if (mw.getCategories) {
            yield categories_1.getCategoriesForArticles(stores_1.articleDetailXId, downloader, redis);
            while ((yield categories_1.trimUnmirroredPages(downloader)) > 0) { // Remove unmirrored pages, categories, subCategories
                // trimUnmirroredPages returns number of modified articles
            }
            // while ((await simplifyGraph(downloader)).deletedNodes !== 0) {
            //   // keep simplifying graph
            // }
            // await trimUnmirroredPages(downloader); // TODO: improve simplify graph to remove the need for a second trim
        }
        const filenameDate = (new Date()).toISOString().slice(0, 7);
        // Getting total number of articles from Redis
        Logger_1.default.log(`Total articles found in Redis: ${yield stores_1.articleDetailXId.len()}`);
        const dumps = [];
        for (const dumpFormat of dumpFormats) {
            const dump = new Dump_1.Dump(dumpFormat, {
                tmpDir: tmpDirectory,
                username: mwUsername,
                password: mwPassword,
                outputDirectory,
                mainPage,
                filenamePrefix,
                articleList,
                publisher,
                customZimDescription,
                customZimTags,
                customZimTitle,
                withoutZimFullTextIndex,
                resume,
                minifyHtml,
                keepEmptyParagraphs,
                tags: customZimTags,
                filenameDate
            }, Object.assign(Object.assign({}, mwMetaData), { mainPage }), customProcessor);
            dumps.push(dump);
            Logger_1.default.log(`Doing dump`);
            let shouldSkip = false;
            try {
                dump.checkResume();
            }
            catch (err) {
                shouldSkip = true;
            }
            if (shouldSkip) {
                Logger_1.default.log(`Skipping dump`);
            }
            else {
                try {
                    yield doDump(dump);
                }
                catch (err) {
                    debugger;
                    throw err;
                }
                Logger_1.default.log(`Finished dump`);
            }
        }
        Logger_1.default.log('Closing HTTP agents...');
        Logger_1.default.log('All dumping(s) finished with success.');
        function doDump(dump) {
            return __awaiter(this, void 0, void 0, function* () {
                const outZim = path.resolve(dump.opts.outputDirectory, dump.computeFilenameRadical() + '.zim');
                Logger_1.default.log(`Writing zim to [${outZim}]`);
                dump.outFile = outZim;
                const zimCreator = new libzim_1.ZimCreator({
                    fileName: outZim,
                    fullTextIndexLanguage: dump.opts.withoutZimFullTextIndex ? '' : dump.mwMetaData.langIso3,
                    welcome: (dump.opts.mainPage ? dump.opts.mainPage : 'index'),
                    compression: zstd ? 'zstd' : 'lzma',
                }, {
                    Tags: dump.computeZimTags(),
                    Language: dump.mwMetaData.langIso3,
                    Title: dump.opts.customZimTitle || dump.mwMetaData.title,
                    Name: dump.computeFilenameRadical(false, true, true),
                    Flavour: dump.computeFlavour(),
                    Description: dump.opts.customZimDescription || dump.mwMetaData.subTitle,
                    Creator: dump.mwMetaData.creator,
                    Publisher: dump.opts.publisher,
                });
                const scraperArticle = new libzim_1.ZimArticle({
                    ns: 'M',
                    data: `mwoffliner ${packageJSON.version}`,
                    url: 'Scraper',
                });
                zimCreator.addArticle(scraperArticle);
                Logger_1.default.info('Copying Static Resource Files');
                yield util_1.saveStaticFiles(config_1.config, zimCreator);
                Logger_1.default.info('Finding stylesheets to download');
                const stylesheetsToGet = yield dump.getRelevantStylesheetUrls(downloader);
                Logger_1.default.log(`Found [${stylesheetsToGet.length}] stylesheets to download`);
                Logger_1.default.log(`Downloading stylesheets and populating media queue`);
                const { finalCss, } = yield util_1.getAndProcessStylesheets(downloader, stylesheetsToGet);
                Logger_1.default.log(`Downloaded stylesheets`);
                const article = new libzim_1.ZimArticle({ url: `${config_1.config.output.dirs.mediawiki}/style.css`, data: finalCss, ns: '-' });
                zimCreator.addArticle(article);
                yield saveFavicon(dump, zimCreator);
                yield getThumbnailsData();
                Logger_1.default.log(`Getting Main Page`);
                yield getMainPage(dump, zimCreator, downloader);
                Logger_1.default.log(`Getting articles`);
                const { jsModuleDependencies, cssModuleDependencies } = yield saveArticles_1.saveArticles(zimCreator, downloader, mw, dump);
                Logger_1.default.log(`Found [${jsModuleDependencies.size}] js module dependencies`);
                Logger_1.default.log(`Found [${cssModuleDependencies.size}] style module dependencies`);
                const allDependenciesWithType = [
                    { type: 'js', moduleList: Array.from(jsModuleDependencies) },
                    { type: 'css', moduleList: Array.from(cssModuleDependencies) },
                ];
                if (downloader.webp) {
                    Logger_1.default.log('Downloading polyfill module');
                    util_1.importPolyfillModules(zimCreator);
                }
                Logger_1.default.log(`Downloading module dependencies`);
                yield Promise.all(allDependenciesWithType.map(({ type, moduleList }) => __awaiter(this, void 0, void 0, function* () {
                    return yield p_map_1.default(moduleList, (oneModule) => {
                        return util_1.downloadAndSaveModule(zimCreator, mw, downloader, dump, oneModule, type);
                    }, { concurrency: downloader.speed });
                })));
                yield saveArticles_1.downloadFiles(stores_1.filesToDownloadXPath, zimCreator, dump, downloader);
                Logger_1.default.log(`Flushing Redis file store`);
                yield stores_1.filesToDownloadXPath.flush();
                yield stores_1.filesToRetryXPath.flush();
                Logger_1.default.log(`Writing Article Redirects`);
                yield writeArticleRedirects(downloader, dump, zimCreator);
                Logger_1.default.log(`Finishing Zim Creation`);
                yield zimCreator.finalise();
                Logger_1.default.log(`Summary of scrape actions:`, JSON.stringify(dump.status, null, '\t'));
            });
        }
        /* ********************************* */
        /* FUNCTIONS *********************** */
        /* ********************************* */
        function writeArticleRedirects(downloader, dump, zimCreator) {
            return __awaiter(this, void 0, void 0, function* () {
                yield stores_1.redirectsXId.iterateItems(downloader.speed, (redirects) => __awaiter(this, void 0, void 0, function* () {
                    for (const [redirectId, { targetId, title }] of Object.entries(redirects)) {
                        if (redirectId !== targetId) {
                            const redirectArticle = new libzim_1.ZimArticle({
                                url: redirectId,
                                shouldIndex: true,
                                data: '',
                                ns: 'A',
                                mimeType: 'text/html',
                                // We fake a title, by just removing the underscores
                                title: String(redirectId).replace(/_/g, ' '),
                                redirectUrl: targetId,
                            });
                            zimCreator.addArticle(redirectArticle);
                            dump.status.redirects.written += 1;
                        }
                    }
                }));
            });
        }
        function saveFavicon(dump, zimCreator) {
            return __awaiter(this, void 0, void 0, function* () {
                Logger_1.default.log('Saving favicon.png...');
                function saveFavicon(zimCreator, faviconPath) {
                    return __awaiter(this, void 0, void 0, function* () {
                        try {
                            const source = yield fs_1.default.promises.readFile(faviconPath);
                            const data = yield sharp_1.default(source).resize(48, 48).toBuffer();
                            const article = new libzim_1.ZimArticle({ url: 'favicon', mimeType: 'image/png', data, ns: '-' });
                            return zimCreator.addArticle(article);
                        }
                        catch (e) {
                            throw new Error('Failed to save favicon using sharp');
                        }
                    });
                }
                if (customZimFavicon) {
                    return yield saveFavicon(zimCreator, customZimFavicon);
                }
                const body = yield downloader.getJSON(mw.siteInfoUrl());
                const entries = body.query.general;
                if (!entries.logo) {
                    throw new Error(`********\nNo site Logo Url. Expected a string, but got [${entries.logo}].\n\nPlease try specifying a customZimFavicon (--customZimFavicon=./path/to/your/file.ico)\n********`);
                }
                const parsedUrl = url_1.default.parse(entries.logo);
                const ext = parsedUrl.pathname.split('.').slice(-1)[0];
                const faviconPath = path.join(tmpDirectory, `favicon.${ext}`);
                const logoUrl = parsedUrl.protocol ? entries.logo : 'http:' + entries.logo;
                const logoContent = yield downloader.downloadContent(logoUrl);
                yield util_1.writeFilePromise(faviconPath, logoContent.content, null);
                return yield saveFavicon(zimCreator, faviconPath);
            });
        }
        function getMainPage(dump, zimCreator, downloader) {
            function createMainPage() {
                return __awaiter(this, void 0, void 0, function* () {
                    Logger_1.default.log('Creating main page...');
                    const doc = domino_1.default.createDocument(Templates_1.articleListHomeTemplate
                        .replace('</head>', util_1.genHeaderCSSLink(config_1.config, 'mobile_main_page', dump.mwMetaData.mainPage) + '\n' +
                        util_1.genHeaderCSSLink(config_1.config, 'style', dump.mwMetaData.mainPage) + '\n' +
                        util_1.genHeaderScript(config_1.config, 'images_loaded.min', dump.mwMetaData.mainPage) + '\n' +
                        util_1.genHeaderScript(config_1.config, 'masonry.min', dump.mwMetaData.mainPage) + '\n' +
                        util_1.genHeaderScript(config_1.config, 'article_list_home', dump.mwMetaData.mainPage) + '\n' +
                        util_1.genCanonicalLink(config_1.config, dump.mwMetaData.webUrl, dump.mwMetaData.mainPage) + '\n' +
                        '\n</head>'));
                    doc.querySelector('title').innerHTML = util_1.sanitizeString(dump.mwMetaData.title) || util_1.sanitizeString(dump.opts.customZimTitle);
                    const articlesWithImages = [];
                    const allArticles = [];
                    for (const articleId of articleListLines) {
                        const articleDetail = yield stores_1.articleDetailXId.get(articleId);
                        if (articleDetail) {
                            allArticles.push(articleDetail);
                            if (articleDetail.thumbnail) {
                                articlesWithImages.push(articleDetail);
                                if (articlesWithImages.length >= 100) {
                                    break;
                                }
                            }
                        }
                    }
                    if (articlesWithImages.length > util_1.MIN_IMAGE_THRESHOLD_ARTICLELIST_PAGE) {
                        const articlesWithImagesEl = articlesWithImages.map((article) => util_1.makeArticleImageTile(dump, article, downloader.webp)).join('\n');
                        doc.body.innerHTML = `<div id='container'><div id='content'>${articlesWithImagesEl}</div></div>`;
                    }
                    else {
                        const articlesWithoutImagesEl = allArticles.map((article) => util_1.makeArticleListItem(dump, article)).join('\n');
                        doc.body.innerHTML = `<ul id='list'>${articlesWithoutImagesEl}</ul>`;
                    }
                    /* Write the static html file */
                    const article = new libzim_1.ZimArticle({ url: 'index', data: doc.documentElement.outerHTML, ns: 'A', mimeType: 'text/html', title: 'Main Page' });
                    return zimCreator.addArticle(article);
                });
            }
            function createMainPageRedirect() {
                Logger_1.default.log(`Create main page redirection from [index] to [${'A/' + mainPage}]`);
                const article = new libzim_1.ZimArticle({
                    url: 'index',
                    shouldIndex: true,
                    data: '',
                    ns: 'A',
                    mimeType: 'text/html',
                    title: mainPage,
                    redirectUrl: mainPage,
                });
                return zimCreator.addArticle(article);
            }
            return mainPage ? createMainPageRedirect() : createMainPage();
        }
        function getThumbnailsData() {
            return __awaiter(this, void 0, void 0, function* () {
                if (customMainPage || !articleList || articleListLines.length <= util_1.MIN_IMAGE_THRESHOLD_ARTICLELIST_PAGE)
                    return;
                Logger_1.default.log(`Updating article thumbnails for articles`);
                let articleIndex = 0;
                let articlesWithImages = 0;
                while (articleIndex < articleListLines.length && articlesWithImages <= 100) {
                    const articleId = articleListLines[articleIndex];
                    articleIndex++;
                    try {
                        const articleDetail = yield stores_1.articleDetailXId.get(articleId);
                        if (!articleDetail)
                            continue;
                        const imageUrl = articleDetail.thumbnail;
                        if (!imageUrl)
                            continue;
                        const { width: oldWidth } = util_1.getSizeFromUrl(imageUrl.source);
                        const suitableResUrl = imageUrl.source
                            .replace(`/${oldWidth}px-`, '/500px-')
                            .replace(`-${oldWidth}px-`, '-500px-');
                        const { mult, width } = util_1.getSizeFromUrl(suitableResUrl);
                        const path = util_1.getMediaBase(suitableResUrl, false);
                        articleDetail.internalThumbnailUrl = util_1.getRelativeFilePath('Main_Page', util_1.getMediaBase(suitableResUrl, true), 'I');
                        yield Promise.all([
                            stores_1.filesToDownloadXPath.set(path, { url: downloader.serializeUrl(suitableResUrl), mult, width }),
                            stores_1.articleDetailXId.set(articleId, articleDetail)
                        ]);
                        articlesWithImages++;
                    }
                    catch (err) {
                        Logger_1.default.warn(`Failed to parse thumbnail for [${articleId}], skipping...`);
                    }
                }
            });
        }
        closeRedis(redis);
        return dumps;
    });
}
exports.execute = execute;
//# sourceMappingURL=mwoffliner.lib.js.map