"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sanitize_customFlavour = exports.sanitize_customZimFavicon = exports.sanitize_redis = exports.sanitize_adminEmail = exports.sanitize_mwUrl = exports.sanitize_speed = exports.sanitize_all = void 0;
const os_1 = __importDefault(require("os"));
const S3_1 = __importDefault(require("./S3"));
const axios_1 = __importDefault(require("axios"));
const Redis_1 = __importDefault(require("./Redis"));
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const Logger_1 = __importDefault(require("./Logger"));
const config_1 = require("./config");
const fs_1 = __importDefault(require("fs"));
const QueryStringParser = __importStar(require("querystring"));
const util_1 = require("./util");
function sanitize_all(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        // extracting all arguments
        const { speed: _speed, adminEmail, mwUrl, customZimFavicon, optimisationCacheUrl, mwWikiPath, } = argv;
        const cpuCount = os_1.default.cpus().length;
        // sanitizing speed
        sanitize_speed(_speed);
        // sanitizing custom flavour
        if (argv.customFlavour) {
            argv.customFlavour = sanitize_customFlavour(argv.customFlavour);
            if (!argv.customFlavour) {
                throw new Error('Custom Flavour not found');
            }
        }
        // sanitizing S3
        try {
            if (optimisationCacheUrl) {
                // Decompose the url with path and other S3 creds
                const s3UrlObj = url_1.default.parse(optimisationCacheUrl);
                const queryReader = QueryStringParser.parse(s3UrlObj.query);
                const s3Url = (s3UrlObj.protocol || 'https:') + '//' + (s3UrlObj.host || '') + (s3UrlObj.pathname || '');
                this.s3Obj = new S3_1.default(s3Url, queryReader);
                yield this.s3Obj.initialise().then(() => {
                    Logger_1.default.log('Successfully logged in S3');
                });
            }
        }
        catch (err) {
            throw err;
        }
        // sanitizing mwUrl
        yield sanitize_mwUrl(mwUrl).catch((err) => {
            throw err;
        });
        // sanitize Custom Main Page
        if (argv.customMainPage) {
            argv.customMainPage = argv.customMainPage.replace(/ /g, '_');
        }
        // sanitizing adminEmail
        sanitize_adminEmail(adminEmail);
        // Redis client sanitization
        // created a redis client and then closed it.
        sanitize_redis(argv);
        // sanitizing custom zim favicon
        if (customZimFavicon) {
            yield sanitize_customZimFavicon(customZimFavicon)
                .catch(err => {
                throw err;
            });
        }
    });
}
exports.sanitize_all = sanitize_all;
function sanitize_speed(_speed) {
    if (_speed && isNaN(_speed)) {
        throw new Error(`speed is not a number, please give a number value to --speed.`);
    }
}
exports.sanitize_speed = sanitize_speed;
function sanitize_mwUrl(mwUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        yield axios_1.default
            .get(mwUrl)
            .catch((err) => {
            throw new Error(`mwUrl [${mwUrl}] is not valid.`);
        });
    });
}
exports.sanitize_mwUrl = sanitize_mwUrl;
function sanitize_adminEmail(adminEmail) {
    if (!util_1.isValidEmail(adminEmail)) {
        throw new Error(`Admin email [${adminEmail}] is not valid.`);
    }
}
exports.sanitize_adminEmail = sanitize_adminEmail;
function sanitize_redis(argv) {
    try {
        const sanitize_redis = new Redis_1.default(argv, config_1.config);
        Logger_1.default.log('closing sanitize redis DB');
        sanitize_redis.client.quit();
    }
    catch (err) {
        throw err;
    }
}
exports.sanitize_redis = sanitize_redis;
function sanitize_customZimFavicon(customZimFavicon) {
    return __awaiter(this, void 0, void 0, function* () {
        const faviconIsRemote = customZimFavicon.includes('http');
        if (faviconIsRemote) {
            // make a download to check custom favicon link is valid
            yield axios_1.default
                .get(customZimFavicon)
                .catch((err) => {
                throw new Error(`Failed to download custom zim favicon from [${customZimFavicon}]`);
            });
        }
        else {
            try {
                fs_1.default.readFileSync(customZimFavicon);
            }
            catch (err) {
                throw err;
            }
        }
    });
}
exports.sanitize_customZimFavicon = sanitize_customZimFavicon;
/**
 * Search for the customFlavour in the following order
 *
 * 1. Current directory in which command has been run
 * 2. mwoffliner's extensions directory
 * 3. absolute path(for root folder)
 *
 * Note: CustomFlavour doesn't necessarily need be given with extension(.js)
 * like --customFlavour=wiktionary_fr. Hence, .js is explicitly added for
 * path resolution.
 */
function sanitize_customFlavour(customFlavour) {
    customFlavour += path_1.default.extname(customFlavour) !== '.js' ? '.js' : '';
    return [
        path_1.default.resolve(customFlavour),
        path_1.default.resolve(__dirname, `../extensions/${customFlavour}`),
        customFlavour,
    ].find(function (possiblePath) {
        return fs_1.default.existsSync(possiblePath);
    }) || null;
}
exports.sanitize_customFlavour = sanitize_customFlavour;
//# sourceMappingURL=sanitize-argument.js.map