"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.importPolyfillModules = exports.downloadAndSaveModule = exports.getAndProcessStylesheets = void 0;
const url_1 = __importDefault(require("url"));
const pathParser = __importStar(require("path"));
const async_1 = __importDefault(require("async"));
const Logger_1 = __importDefault(require("../Logger"));
const axios_1 = __importDefault(require("axios"));
const _1 = require(".");
const config_1 = require("../config");
const libzim_1 = require("@openzim/libzim");
const stores_1 = require("../stores");
const fs_1 = __importDefault(require("fs"));
const const_1 = require("./const");
function getAndProcessStylesheets(downloader, links) {
    return __awaiter(this, void 0, void 0, function* () {
        let finalCss = '';
        const stylesheetQueue = async_1.default.queue((link, finished) => __awaiter(this, void 0, void 0, function* () {
            const cssUrl = typeof link === 'object' ? _1.getFullUrl(link.getAttribute('href'), downloader.mw.baseUrl) : link;
            const linkMedia = typeof link === 'object' ? link.getAttribute('media') : null;
            try {
                /* link might be a 'link' DOM node or an URL */
                if (cssUrl && !cssUrl.match('^data')) {
                    const cssUrlRegexp = new RegExp('url\\([\'"]{0,1}(.+?)[\'"]{0,1}\\)', 'gi');
                    Logger_1.default.info(`Downloading CSS from ${decodeURI(cssUrl)}`);
                    const { content } = yield downloader.downloadContent(cssUrl);
                    const body = content.toString();
                    let rewrittenCss = `\n/* start ${cssUrl} */\n\n`;
                    rewrittenCss += linkMedia ? `@media ${linkMedia}  {\n` : '\n';
                    rewrittenCss += `${body}\n`;
                    rewrittenCss += linkMedia ? `} /* @media ${linkMedia} */\n` : '\n';
                    rewrittenCss += `\n/* end   ${cssUrl} */\n`;
                    /* Downloading CSS dependencies */
                    let match;
                    // tslint:disable-next-line:no-conditional-assignment
                    while ((match = cssUrlRegexp.exec(body))) {
                        let url = match[1];
                        /* Avoid 'data', so no url dependency */
                        if (!url.match('^data')) {
                            const filePathname = url_1.default.parse(url, false, true).pathname;
                            if (filePathname) {
                                const filename = pathParser.basename(filePathname).replace(/-.*x./, '.');
                                /* Rewrite the CSS */
                                rewrittenCss = rewrittenCss.replace(url, filename);
                                /* Need a rewrite if url doesn't include protocol */
                                url = _1.getFullUrl(url, cssUrl);
                                url = url.indexOf('%') < 0 ? encodeURI(url) : url;
                                /* Download CSS dependency, but avoid duplicate calls */
                                if (!downloader.cssDependenceUrls.hasOwnProperty(url) && filename) {
                                    downloader.cssDependenceUrls[url] = true;
                                    stores_1.filesToDownloadXPath.set(config_1.config.output.dirs.mediawiki + '/' + filename, { url: downloader.serializeUrl(url), namespace: '-' });
                                }
                            }
                            else {
                                Logger_1.default.warn(`Skipping CSS [url(${url})] because the pathname could not be found [${filePathname}]`);
                            }
                        }
                    }
                    finalCss += rewrittenCss;
                    finished();
                }
            }
            catch (err) {
                Logger_1.default.warn(`Failed to get CSS from [${cssUrl}]`);
                finished();
            }
        }), Number(downloader.speed));
        stylesheetQueue.push(links);
        return new Promise((resolve, reject) => {
            stylesheetQueue.drain(resolve);
        }).then(() => {
            return {
                finalCss,
            };
        });
    });
}
exports.getAndProcessStylesheets = getAndProcessStylesheets;
function downloadAndSaveModule(zimCreator, mw, downloader, dump, module, type) {
    return __awaiter(this, void 0, void 0, function* () {
        // param :
        //   module : string : the name of the module
        //   moduleUri : string : the path where the module will be saved into the zim
        //   type : string : either 'js' or 'css'
        // this function save a key into Redis db in the form of module.type -> moduleUri
        // return :
        //   a promise resolving 1 if data has been succesfully saved or resolving 0 if data was already in Redis
        // the function hackStartupModule changes startup script by returning true for all modules so that load.php is not called.
        // it also removes requestIdleCallback as in our case window is idle after all script tags are called but those script tags
        // will require the functions which would have been loaded by doPropagation.
        function hackStartUpModule(jsCode) {
            if (!const_1.ALL_READY_FUNCTION.test(jsCode) || !const_1.DO_PROPAGATION.test(jsCode)) {
                throw new Error('unable to hack startup module');
            }
            return jsCode.replace(const_1.DO_PROPAGATION, 'doPropagation();')
                .replace(const_1.ALL_READY_FUNCTION, 'function allReady( modules ) { return true;');
        }
        let apiParameterOnly;
        if (type === 'js') {
            apiParameterOnly = 'scripts';
        }
        else if (type === 'css') {
            apiParameterOnly = 'styles';
        }
        const moduleApiUrl = encodeURI(`${mw.modulePath}debug=true&lang=en&modules=${module}&only=${apiParameterOnly}&skin=vector&version=&*`);
        Logger_1.default.info(`Getting [${type}] module [${moduleApiUrl}]`);
        const { content } = yield downloader.downloadContent(moduleApiUrl);
        let text = content.toString();
        if (module === 'startup' && type === 'js') {
            text = hackStartUpModule(text);
        }
        try {
            const articleId = type === 'js'
                ? _1.jsPath(module, config_1.config.output.dirs.mediawiki)
                : _1.cssPath(module, config_1.config.output.dirs.mediawiki);
            const article = new libzim_1.ZimArticle({ url: articleId, data: text, ns: '-' });
            zimCreator.addArticle(article);
            Logger_1.default.info(`Saved module [${module}]`);
        }
        catch (e) {
            Logger_1.default.error(`Failed to get module with url [${moduleApiUrl}]\nYou may need to specify a custom --mwModulePath`, e);
            throw e;
        }
    });
}
exports.downloadAndSaveModule = downloadAndSaveModule;
// URLs should be kept the same as Kiwix JS relies on it.
function importPolyfillModules(zimCreator) {
    return __awaiter(this, void 0, void 0, function* () {
        [
            { name: 'webpHeroPolyfill', path: 'webp-hero/dist-cjs/polyfills.js' },
            { name: 'webpHeroBundle', path: 'webp-hero/dist-cjs/webp-hero.bundle.js' }
        ].forEach(({ name, path }) => {
            const article = new libzim_1.ZimArticle({
                url: _1.jsPath(name),
                data: fs_1.default.readFileSync(require.resolve(path), 'utf8').toString(),
                ns: '-'
            });
            zimCreator.addArticle(article);
        });
        const content = yield axios_1.default.get(const_1.WEBP_HANDLER_URL, { responseType: 'arraybuffer', timeout: 60000, validateStatus(status) { return ([200, 302, 304].indexOf(status) > -1); } })
            .then((a) => a.data)
            .catch((err) => {
            throw new Error(`Failed to download webpHandler from [${const_1.WEBP_HANDLER_URL}]: ${err}`);
        });
        const article = new libzim_1.ZimArticle({
            url: _1.jsPath('webpHandler'),
            data: content,
            ns: '-'
        });
        zimCreator.addArticle(article);
    });
}
exports.importPolyfillModules = importPolyfillModules;
//# sourceMappingURL=dump.js.map