"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isWebpCandidateImageMimeType = exports.isBitmapImageMimeType = exports.isImageMimeType = exports.isWebpCandidateImageUrl = exports.isImageUrl = exports.stripHttpFromUrl = exports.ensureTrailingChar = exports.encodeArticleIdForZimHtmlUrl = exports.sanitizeString = exports.objToQueryString = exports.isNodeModule = exports.normalizeModule = exports.getRelativeFilePath = exports.deDup = exports.zip = exports.getStrippedTitleFromHtml = exports.getMediaBase = exports.getIso3 = exports.getDumps = exports.genCanonicalLink = exports.genHeaderScript = exports.genHeaderCSSLink = exports.jsPath = exports.cssPath = exports.saveStaticFiles = exports.interpolateTranslationString = exports.getStringsForLang = exports.migrateChildren = exports.contains = exports.readFilePromise = exports.writeFilePromise = exports.mkdirPromise = exports.randomString = exports.getSizeFromUrl = exports.getFullUrl = exports.touch = exports.decodeURIComponent = exports.ucFirst = exports.lcFirst = exports.isValidEmail = void 0;
const crypto_1 = __importDefault(require("crypto"));
const domino_1 = __importDefault(require("domino"));
const country_language_1 = __importDefault(require("country-language"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_2 = __importDefault(require("path"));
const libzim_1 = require("@openzim/libzim");
const config_1 = require("../config");
const Logger_1 = __importDefault(require("../Logger"));
const const_1 = require("./const");
function isValidEmail(email) {
    const emailRegex = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return emailRegex.test(email);
}
exports.isValidEmail = isValidEmail;
function lcFirst(str) {
    str += '';
    const f = str.charAt(0).toLowerCase();
    return f + str.substr(1);
}
exports.lcFirst = lcFirst;
function ucFirst(str) {
    str += '';
    const f = str.charAt(0).toUpperCase();
    return f + str.substr(1);
}
exports.ucFirst = ucFirst;
function _decodeURIComponent(uri) {
    try {
        return decodeURIComponent(uri);
    }
    catch (error) {
        Logger_1.default.warn(error);
        return uri;
    }
}
exports.decodeURIComponent = _decodeURIComponent;
function touch(paths) {
    const currentDate = Date.now();
    paths = paths instanceof Array ? paths : [paths];
    paths.forEach((path) => {
        fs_1.default.utimes(path, currentDate, currentDate, () => null);
    });
}
exports.touch = touch;
function getFullUrl(url, baseUrl) {
    return new URL(url, baseUrl).toString();
}
exports.getFullUrl = getFullUrl;
function getSizeFromUrl(url) {
    let mult;
    let width;
    const widthMatch = url.match(/[\/-]([0-9]+)px-/);
    if (widthMatch) {
        width = Number(widthMatch[1]);
    }
    else {
        const multMatch = url.match(/-([0-9.]+)x\./);
        if (multMatch) {
            mult = Number(multMatch[1]);
        }
    }
    return { mult, width };
}
exports.getSizeFromUrl = getSizeFromUrl;
function randomString(len) {
    let str = '';
    const charSet = 'abcdefghijklmnopqrstuvwxyz0123456789';
    for (let i = 0; i < len; i += 1) {
        const randomPoz = Math.floor(Math.random() * charSet.length);
        str += charSet.substring(randomPoz, randomPoz + 1);
    }
    return str;
}
exports.randomString = randomString;
function mkdirPromise(path) {
    try {
        return mkdirp_1.default(path, { recursive: true });
    }
    catch (err) {
        return err;
    }
}
exports.mkdirPromise = mkdirPromise;
function writeFilePromise(path, content, encoding = 'utf8') {
    return new Promise((resolve, reject) => {
        fs_1.default.writeFile(path, content, encoding, (err) => {
            if (err) {
                reject(err);
            }
            else {
                resolve();
            }
        });
    });
}
exports.writeFilePromise = writeFilePromise;
function readFilePromise(path, encoding = 'utf8') {
    return new Promise((resolve, reject) => {
        fs_1.default.readFile(path, encoding, (err, content) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(content);
            }
        });
    });
}
exports.readFilePromise = readFilePromise;
function contains(arr, value) {
    return arr.some((v) => v === value);
}
exports.contains = contains;
/*
 * Move 'from'.childNodes to 'to' adding them before 'beforeNode'
 * If 'beforeNode' is null, the nodes are appended at the end.
 */
function migrateChildren(from, to, beforeNode) {
    if (beforeNode === undefined) {
        beforeNode = null;
    }
    while (from.firstChild) {
        to.insertBefore(from.firstChild, beforeNode);
    }
}
exports.migrateChildren = migrateChildren;
function getStringsForLang(language, fallbackLanguage = 'en') {
    let strings = {};
    try {
        strings = require(`../../translation/${language}.json`);
    }
    catch (err) {
        Logger_1.default.warn(`Couldn't find strings file for [${language}], falling back to [${fallbackLanguage}]`);
        strings = require(`../../translation/${fallbackLanguage}.json`);
    }
    return strings;
}
exports.getStringsForLang = getStringsForLang;
function interpolateTranslationString(str, parameters) {
    let newString = str;
    for (const key of Object.keys(parameters)) {
        newString = newString.replace(new RegExp(`\\$\\{${key}\\}`, 'g'), parameters[key]);
    }
    return newString;
}
exports.interpolateTranslationString = interpolateTranslationString;
function saveStaticFiles(config, zimCreator) {
    const cssPromises = config.output.cssResources
        .concat(config.output.mainPageCssResources)
        .map((css) => __awaiter(this, void 0, void 0, function* () {
        try {
            const cssCont = yield readFilePromise(path_2.default.resolve(__dirname, `../../res/${css}.css`));
            const article = new libzim_1.ZimArticle({ url: cssPath(css), data: cssCont, ns: '-' });
            zimCreator.addArticle(article);
        }
        catch (error) {
            Logger_1.default.warn(`Could not create ${css} file : ${error}`);
        }
    }));
    const jsPromises = config.output.jsResources.map((js) => __awaiter(this, void 0, void 0, function* () {
        try {
            const jsCont = yield readFilePromise(path_2.default.resolve(__dirname, `../../res/${js}.js`));
            const article = new libzim_1.ZimArticle({ url: jsPath(js), data: jsCont, ns: '-' });
            zimCreator.addArticle(article);
        }
        catch (error) {
            Logger_1.default.warn(`Could not create ${js} file : ${error}`);
        }
    }));
    return Promise.all([
        ...cssPromises,
        ...jsPromises,
    ]);
}
exports.saveStaticFiles = saveStaticFiles;
function cssPath(css, subDirectory = '') {
    return `${subDirectory ? `${subDirectory}/` : ''}${css.replace(/(\.css)?$/, '')}.css`;
}
exports.cssPath = cssPath;
function jsPath(js, subDirectory = '') {
    const path = (isNodeModule(js)) ? normalizeModule(js) : js;
    return `${subDirectory ? `${config_1.config.output.dirs.mediawiki}/` : ''}${path.replace(/(\.js)?$/, '')}.js`;
}
exports.jsPath = jsPath;
function genHeaderCSSLink(config, css, articleId, subDirectory = '') {
    const resourceNamespace = '-';
    const slashesInUrl = articleId.split('/').length - 1;
    const upStr = '../'.repeat(slashesInUrl + 1);
    return `<link href="${upStr}${resourceNamespace}/${cssPath(css, subDirectory)}" rel="stylesheet" type="text/css"/>`;
}
exports.genHeaderCSSLink = genHeaderCSSLink;
function genHeaderScript(config, js, articleId, subDirectory = '') {
    const resourceNamespace = '-';
    const slashesInUrl = articleId.split('/').length - 1;
    const upStr = '../'.repeat(slashesInUrl + 1);
    const path = (isNodeModule(js)) ? normalizeModule(js) : js;
    return `<script src="${upStr}${resourceNamespace}/${jsPath(path, subDirectory)}"></script>`;
}
exports.genHeaderScript = genHeaderScript;
function genCanonicalLink(config, webUrl, articleId) {
    return `<link rel="canonical" href="${webUrl}${encodeURIComponent(articleId)}" />`;
}
exports.genCanonicalLink = genCanonicalLink;
function getDumps(format) {
    let dumps;
    if (format) {
        if (format instanceof Array) {
            dumps = [];
            const self = format.forEach((value) => {
                dumps.push(value === true ? '' : value);
            });
        }
        else if (format !== true) {
            dumps = [format];
        }
    }
    else {
        dumps = [''];
    }
    return dumps;
}
exports.getDumps = getDumps;
function getIso3(langIso2) {
    return new Promise((resolve, reject) => {
        country_language_1.default.getLanguage(langIso2, (error, language) => {
            if (error || !language.iso639_3) {
                reject(error);
            }
            else {
                resolve(language.iso639_3);
            }
        });
    });
}
exports.getIso3 = getIso3;
/* Internal path/url functions */
function getMediaBase(url, escape) {
    const decodedUrl = decodeURI(url);
    let parts;
    let filename;
    // Image thumbs
    if ((parts = const_1.IMAGE_THUMB_URL_REGEX.exec(decodedUrl)) !== null) {
        // Remove trailing / in parts[1] if possible
        parts[1] = parts[1] ? parts[1].substring(0, parts[1].length - 1) : '';
        // Most common case
        if (!parts[1] || parts[1].length <= parts[3].length) {
            filename = parts[3];
        }
        // To handle /...px-thumbnail.jpg use case
        else {
            filename = parts[1] + (parts[4] || '');
        }
    }
    // Latex (equations)
    else if ((parts = const_1.LATEX_IMAGE_URL_REGEX.exec(decodedUrl)) !== null) {
        filename = parts[1] + '.svg';
    }
    // WikiHiero hieroglyphs (betting there won't be a name conflict with main namespace pictures)
    else if ((parts = const_1.WIKIHIERO_IMAGE_URL_REGEX.exec(decodedUrl)) !== null) {
        filename = parts[1];
    }
    // Default behaviour (make a hash of the URL)
    else {
        filename = crypto_1.default.createHash('md5').update(decodedUrl).digest('hex') + path_1.default.extname((new URL(url)).pathname);
    }
    return escape ? encodeURIComponent(filename) : filename;
}
exports.getMediaBase = getMediaBase;
function getStrippedTitleFromHtml(html) {
    const doc = domino_1.default.createDocument(html);
    const titleEl = doc.querySelector('title');
    if (titleEl) {
        return titleEl.textContent;
    }
    else {
        return '';
    }
}
exports.getStrippedTitleFromHtml = getStrippedTitleFromHtml;
function zip(...args) {
    const len = Math.max(...args.map((arr) => arr.length));
    return ','.repeat(len).split(',')
        .map((_, i) => {
        return args.map((arr) => arr[i]);
    });
}
exports.zip = zip;
function deDup(_arr, getter) {
    const arr = _arr.sort((a, b) => getter(a) < getter(b) ? -1 : 1);
    return arr.filter((item, index, arr) => {
        if (index + 1 === arr.length) {
            return true;
        }
        return getter(item) !== getter(arr[index + 1]);
    });
}
exports.deDup = deDup;
function getRelativeFilePath(parentArticleId, fileBase, resourceNamespace) {
    const slashesInUrl = parentArticleId.split('/').length - 1;
    const upStr = '../'.repeat(slashesInUrl + 1);
    const newUrl = `${upStr}${resourceNamespace}/` + fileBase;
    return newUrl;
}
exports.getRelativeFilePath = getRelativeFilePath;
function normalizeModule(path) {
    return path.replace('../node_modules', 'node_module');
}
exports.normalizeModule = normalizeModule;
function isNodeModule(path) {
    return path.startsWith('../node_module');
}
exports.isNodeModule = isNodeModule;
function objToQueryString(obj) {
    const str = [];
    for (const p in obj) {
        if (obj.hasOwnProperty(p) && typeof obj[p] !== 'undefined') {
            str.push(encodeURIComponent(p) + '=' + encodeURIComponent(obj[p]));
        }
    }
    return str.join('&');
}
exports.objToQueryString = objToQueryString;
function sanitizeString(str) {
    return str.replace(/[&<>"'*=//]/g, ' ');
}
exports.sanitizeString = sanitizeString;
// We will need the encoded URL on article load so that we can set the hrefs of anchor tag correctly,
// but we must not encode the '/' character or else relative links may fail
function encodeArticleIdForZimHtmlUrl(articleId) {
    return articleId && encodeURIComponent(articleId.startsWith('/') ? `./${articleId}` : articleId).replace(/%2F/g, '/');
}
exports.encodeArticleIdForZimHtmlUrl = encodeArticleIdForZimHtmlUrl;
function ensureTrailingChar(input, trailingChar) {
    const pattern = `([^\\${trailingChar}])$`;
    const rx = new RegExp(pattern);
    return input.replace(rx, '$1' + trailingChar);
}
exports.ensureTrailingChar = ensureTrailingChar;
function stripHttpFromUrl(url) {
    return url.replace(const_1.FIND_HTTP_REGEX, '');
}
exports.stripHttpFromUrl = stripHttpFromUrl;
function isImageUrl(url) {
    return const_1.IMAGE_URL_REGEX.test(url);
}
exports.isImageUrl = isImageUrl;
function isWebpCandidateImageUrl(url) {
    return const_1.WEBP_CANDIDATE_IMAGE_URL_REGEX.test(url);
}
exports.isWebpCandidateImageUrl = isWebpCandidateImageUrl;
function isImageMimeType(mimeType) {
    return const_1.IMAGE_MIME_REGEX.test(mimeType);
}
exports.isImageMimeType = isImageMimeType;
function isBitmapImageMimeType(mimeType) {
    return const_1.BITMAP_IMAGE_MIME_REGEX.test(mimeType);
}
exports.isBitmapImageMimeType = isBitmapImageMimeType;
function isWebpCandidateImageMimeType(webp, content_type) {
    return webp && const_1.WEBP_CANDIDATE_IMAGE_MIME_TYPE.test(content_type);
}
exports.isWebpCandidateImageMimeType = isWebpCandidateImageMimeType;
//# sourceMappingURL=misc.js.map