"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.rewriteUrl = exports.removeOrEncodeLink = void 0;
const urlParser = __importStar(require("url"));
const saveArticles_1 = require("./saveArticles");
const misc_1 = require("./misc");
const stores_1 = require("../stores");
const DOMUtils_1 = __importDefault(require("../DOMUtils"));
const Logger_1 = __importDefault(require("../Logger"));
function removeOrEncodeLink(mw, dump, linkNode, href) {
    return __awaiter(this, void 0, void 0, function* () {
        const title = mw.extractPageTitleFromHref(href);
        if (!title) {
            return;
        }
        if (yield saveArticles_1.isMirrored(title)) {
            /* Deal with local anchor */
            const localAnchor = href.lastIndexOf('#') === -1 ? '' : href.substr(href.lastIndexOf('#'));
            linkNode.setAttribute('href', misc_1.encodeArticleIdForZimHtmlUrl(title) + localAnchor);
            return;
        }
        else {
            const res = yield stores_1.redirectsXId.get(title);
            if (res) {
                linkNode.setAttribute('href', misc_1.encodeArticleIdForZimHtmlUrl(title));
            }
            else {
                misc_1.migrateChildren(linkNode, linkNode.parentNode, linkNode);
                linkNode.parentNode.removeChild(linkNode);
            }
            return;
        }
    });
}
exports.removeOrEncodeLink = removeOrEncodeLink;
function rewriteUrl(articleId, mw, dump, linkNode) {
    return __awaiter(this, void 0, void 0, function* () {
        const mediaDependencies = [];
        let rel = linkNode.getAttribute('rel');
        let href = linkNode.getAttribute('href') || '';
        let hrefProtocol;
        try {
            hrefProtocol = urlParser.parse(href).protocol;
        }
        catch (e) {
            return { mediaDependencies };
        }
        if (hrefProtocol && !hrefProtocol.includes('http')) {
            return { mediaDependencies }; // e.g. geo:11111,11111
        }
        if (rel === 'mwo:NoRewrite') {
            return { mediaDependencies };
        }
        if (!hrefProtocol && href.slice(0, 2) === '//') {
            href = `${mw.webUrl.protocol}${href}`;
            linkNode.setAttribute('href', href);
            hrefProtocol = mw.webUrl.protocol;
        }
        if (!rel && linkNode.getAttribute('resource')) {
            rel = 'mw:MediaLink';
        }
        if (hrefProtocol && hrefProtocol.includes('http') && !rel) {
            rel = 'mw:ExtLink';
        }
        if (!href) {
            DOMUtils_1.default.deleteNode(linkNode);
            return { mediaDependencies };
        }
        else if (href.substring(0, 1) === '#') {
            return { mediaDependencies };
        }
        else {
            /* Deal with custom geo. URL replacement, for example:
             * http://maps.wikivoyage-ev.org/w/poimap2.php?lat=44.5044943&lon=34.1969633&zoom=15&layer=M&lang=ru&name=%D0%9C%D0%B0%D1%81%D1%81%D0%B0%D0%BD%D0%B4%D1%80%D0%B0
             * http://tools.wmflabs.org/geohack/geohack.php?language=fr&pagename=Tour_Eiffel&params=48.85825_N_2.2945_E_type:landmark_region:fr
             */
            if (rel !== 'mw:WikiLink') {
                let lat;
                let lon;
                if (/poimap2\.php/i.test(href)) {
                    const hrefQuery = urlParser.parse(href, true).query;
                    lat = parseFloat(hrefQuery.lat);
                    lon = parseFloat(hrefQuery.lon);
                }
                else if (/geohack\.php/i.test(href)) {
                    let { params } = urlParser.parse(href, true).query;
                    /* "params" might be an array, try to detect the geo localization one */
                    if (params instanceof Array) {
                        let i = 0;
                        while (params[i] && isNaN(+params[i][0])) {
                            i += 1;
                        }
                        params = params[i];
                    }
                    if (params) {
                        // see https://bitbucket.org/magnusmanske/geohack/src public_html geo_param.php
                        const pieces = params.toUpperCase().split('_');
                        const semiPieces = pieces.length > 0 ? pieces[0].split(';') : undefined;
                        if (semiPieces && semiPieces.length === 2) {
                            [lat, lon] = semiPieces;
                        }
                        else {
                            const factors = [1, 60, 3600];
                            let offs = 0;
                            const deg = (hemiHash) => {
                                let out = 0;
                                let hemiSign = 0;
                                for (let i = 0; i < 4 && i + offs < pieces.length; i += 1) {
                                    const v = pieces[i + offs];
                                    hemiSign = hemiHash[v];
                                    if (hemiSign) {
                                        offs = i + 1;
                                        break;
                                    }
                                    out += +v / factors[i];
                                }
                                return out * hemiSign;
                            };
                            lat = deg({ N: 1, S: -1 });
                            lon = deg({ E: 1, W: -1, O: 1 });
                        }
                    }
                }
                else if (/Special:Map/i.test(href)) {
                    const parts = href.split('/');
                    lat = parts[4];
                    lon = parts[5];
                }
                else if (rel === 'mw:MediaLink') {
                    const shouldScrape = href.includes('.pdf') && !dump.nopdf ||
                        href.includes('.ogg') && !dump.nopic && !dump.novid && !dump.nodet;
                    if (shouldScrape) {
                        try {
                            const newHref = misc_1.getRelativeFilePath(articleId, misc_1.getMediaBase(href, true), 'I');
                            linkNode.setAttribute('href', newHref);
                            mediaDependencies.push(href);
                        }
                        catch (err) {
                            Logger_1.default.warn('Error parsing url:', err);
                            DOMUtils_1.default.deleteNode(linkNode);
                        }
                    }
                }
                if (!isNaN(lat) && !isNaN(lon)) {
                    href = `geo:${lat},${lon}`;
                    linkNode.setAttribute('href', href);
                    return { mediaDependencies };
                }
            }
            if (rel) { // This is Parsoid HTML
                /* Add 'external' class to interwiki links */
                if (rel.substring(0, 10) === 'mw:ExtLink' || rel === 'mw:WikiLink/Interwiki') {
                    DOMUtils_1.default.appendToAttr(linkNode, 'class', 'external');
                }
                /* Check if the link is "valid" */
                if (!href) {
                    throw new Error(`No href attribute in the following code, in article\n${linkNode.outerHTML}`);
                }
                /* Rewrite external links starting with // */
                if (rel.substring(0, 10) === 'mw:ExtLink' || rel === 'nofollow') {
                    if (href.substring(0, 1) === '/') {
                        linkNode.setAttribute('href', misc_1.getFullUrl(href, mw.baseUrl));
                    }
                    else if (href.substring(0, 2) === './') {
                        misc_1.migrateChildren(linkNode, linkNode.parentNode, linkNode);
                        linkNode.parentNode.removeChild(linkNode);
                    }
                    return { mediaDependencies };
                }
                else if (rel === 'mw:WikiLink' || rel === 'mw:referencedBy') {
                    yield removeOrEncodeLink(mw, dump, linkNode, href);
                }
                else {
                    return { mediaDependencies };
                }
            }
            else { // This is MediaWiki HTML
                yield removeOrEncodeLink(mw, dump, linkNode, href);
            }
            if (articleId.includes('/')) {
                const href = linkNode.getAttribute('href'); // href is modified above, so this is necessary
                const resourceNamespace = 'A';
                const slashesInUrl = articleId.split('/').length - 1;
                const upStr = '../'.repeat(slashesInUrl + 1);
                linkNode.setAttribute('href', `${upStr}${resourceNamespace}/${href}`);
            }
        }
        return { mediaDependencies };
    });
}
exports.rewriteUrl = rewriteUrl;
//# sourceMappingURL=rewriteUrls.js.map