"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMirrored = exports.applyOtherTreatments = exports.treatMedias = exports.treatSubtitle = exports.treatVideo = exports.saveArticles = exports.downloadFiles = void 0;
const Logger_1 = __importDefault(require("../Logger"));
const libzim_1 = require("@openzim/libzim");
const html_minifier_1 = __importDefault(require("html-minifier"));
const QueryStringParser = __importStar(require("querystring"));
const p_map_1 = __importDefault(require("p-map"));
const DOMUtils_1 = __importDefault(require("../DOMUtils"));
const domino = __importStar(require("domino"));
const _1 = require(".");
const config_1 = require("../config");
const Templates_1 = require("../Templates");
const stores_1 = require("../stores");
const misc_1 = require("./misc");
const rewriteUrls_1 = require("./rewriteUrls");
const const_1 = require("./const");
const genericJsModules = config_1.config.output.mw.js;
const genericCssModules = config_1.config.output.mw.css;
function downloadFiles(fileStore, zimCreator, dump, downloader, retryLater = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const filesForAttempt = yield fileStore.len();
        const filesTotal = filesForAttempt + dump.status.files.success + dump.status.files.fail;
        Logger_1.default.log(`${retryLater ? '' : 'RE-'}Downloading a total of [${retryLater ? filesTotal : filesForAttempt}] files...`);
        let prevPercentProgress;
        yield fileStore.iterateItems(downloader.speed, (fileDownloadPairs, workerId) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.info(`Worker [${workerId}] processing batch of [${Object.keys(fileDownloadPairs).length}] files`);
            // todo align fileDownloadPairs and listOfArguments
            const listOfArguments = [];
            for (const [path, { url, namespace, mult, width }] of Object.entries(fileDownloadPairs)) {
                listOfArguments.push({ path, url, namespace, mult, width });
            }
            const responses = yield downloadBulk(listOfArguments, downloader);
            for (const resp of responses) {
                let isFailed = false;
                try {
                    if (resp.result && resp.result.content) {
                        const article = new libzim_1.ZimArticle({
                            url: resp.path,
                            data: resp.result.content,
                            ns: resp.namespace || 'I',
                            mimeType: resp.result.responseHeaders['content-type'],
                        });
                        zimCreator.addArticle(article);
                        dump.status.files.success += 1;
                    }
                    else {
                        isFailed = true;
                    }
                }
                catch (err) {
                    isFailed = true;
                }
                finally {
                    if (isFailed) {
                        // todo don't queue 404 for retry
                        if (retryLater) {
                            yield stores_1.filesToRetryXPath.set(resp.path, { url: resp.url, namespace: resp.namespace, mult: resp.mult, width: resp.width });
                        }
                        else {
                            Logger_1.default.warn(`Error downloading file [${downloader.deserializeUrl(resp.url)}], skipping`);
                            dump.status.files.fail += 1;
                        }
                    }
                }
                if ((dump.status.files.success + dump.status.files.fail) % 10 === 0) {
                    const percentProgress = ((dump.status.files.success + dump.status.files.fail) / filesTotal * 100).toFixed(1);
                    if (percentProgress !== prevPercentProgress) {
                        prevPercentProgress = percentProgress;
                        Logger_1.default.log(`Progress downloading files [${dump.status.files.success + dump.status.files.fail}/${filesTotal}] [${percentProgress}%]`);
                    }
                }
            }
        }));
        if (retryLater) {
            const isThereAnythingToRetry = (yield stores_1.filesToRetryXPath.len()) > 0;
            if (isThereAnythingToRetry) {
                yield downloadFiles(stores_1.filesToRetryXPath, zimCreator, dump, downloader, false);
            }
            else {
                Logger_1.default.log('No files to retry');
            }
        }
        Logger_1.default.log(`Done with ${retryLater ? '' : 'RE-'}Downloading a total of [${retryLater ? filesTotal : filesForAttempt}] files`);
    });
}
exports.downloadFiles = downloadFiles;
function downloadBulk(listOfArguments, downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // Enhance arguments array to have an index of the argument at hand
            const argsCopy = [].concat(listOfArguments.map((val, ind) => ({ val, ind })));
            const argList = [];
            while (argsCopy.length > 0) {
                const arg = argsCopy.shift();
                argList.push(arg);
            }
            return p_map_1.default(argList, (arg) => __awaiter(this, void 0, void 0, function* () {
                const resp = {};
                resp.path = arg.val.path;
                resp.url = arg.val.url;
                resp.namespace = arg.val.namespace;
                resp.mult = arg.val.mult;
                resp.width = arg.val.width;
                return downloader.downloadContent(arg.val.url).then((r) => {
                    resp.result = r;
                    resp.path += resp.result.responseHeaders.path_postfix || '';
                    return resp;
                }).catch((err) => {
                    return resp;
                });
            }), { concurrency: const_1.CONCURRENCY_LIMIT });
        }
        catch (err) {
            Logger_1.default.log(`Not able download in bulk due to ${err}`);
        }
    });
}
function getAllArticlesToKeep(downloader, mw, dump) {
    return __awaiter(this, void 0, void 0, function* () {
        yield stores_1.articleDetailXId.iterateItems(downloader.speed, (articleKeyValuePairs) => __awaiter(this, void 0, void 0, function* () {
            for (const [articleId] of Object.entries(articleKeyValuePairs)) {
                try {
                    const rets = yield downloader.getArticle(articleId, dump);
                    for (const { articleId, html: articleHtml } of rets) {
                        if (!articleHtml) {
                            continue;
                        }
                        const doc = domino.createDocument(articleHtml);
                        if (!dump.isMainPage(articleId) && !(yield dump.customProcessor.shouldKeepArticle(articleId, doc))) {
                            stores_1.articleDetailXId.delete(articleId);
                        }
                    }
                }
                catch (err) {
                    Logger_1.default.warn(`Error downloading article [${articleId}], skipping`, err);
                    stores_1.articleDetailXId.delete(articleId);
                }
            }
        }));
    });
}
function saveArticles(zimCreator, downloader, mw, dump) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const jsModuleDependencies = new Set();
        const cssModuleDependencies = new Set();
        let jsConfigVars = '';
        let prevPercentProgress;
        const articlesTotal = yield stores_1.articleDetailXId.len();
        if ((_a = dump.customProcessor) === null || _a === void 0 ? void 0 : _a.shouldKeepArticle) {
            yield getAllArticlesToKeep(downloader, mw, dump);
        }
        yield stores_1.articleDetailXId.iterateItems(downloader.speed, (articleKeyValuePairs, workerId) => __awaiter(this, void 0, void 0, function* () {
            var _b;
            Logger_1.default.info(`Worker [${workerId}] processing batch of article ids [${Logger_1.default.logifyArray(Object.keys(articleKeyValuePairs))}]`);
            for (const [articleId, articleDetail] of Object.entries(articleKeyValuePairs)) {
                try {
                    const rets = yield downloader.getArticle(articleId, dump);
                    for (const { articleId, displayTitle: articleTitle, html: articleHtml } of rets) {
                        const nonPaginatedArticleId = articleDetail.title;
                        if (!articleHtml) {
                            Logger_1.default.warn(`No HTML returned for article [${articleId}], skipping`);
                            continue;
                        }
                        const { articleDoc: _articleDoc, mediaDependencies, subtitles } = yield processArticleHtml(articleHtml, downloader, mw, dump, articleId);
                        let articleDoc = _articleDoc;
                        if (!dump.isMainPage(articleId) && ((_b = dump.customProcessor) === null || _b === void 0 ? void 0 : _b.preProcessArticle)) {
                            articleDoc = yield dump.customProcessor.preProcessArticle(articleId, articleDoc);
                        }
                        for (const dep of mediaDependencies) {
                            const { mult, width } = misc_1.getSizeFromUrl(dep.url);
                            const existingVal = yield stores_1.filesToDownloadXPath.get(dep.path);
                            const currentDepIsHigherRes = !existingVal || (existingVal.width < (width || 10e6)) || existingVal.mult < (mult || 1);
                            if (currentDepIsHigherRes) {
                                yield stores_1.filesToDownloadXPath.set(dep.path, { url: downloader.serializeUrl(dep.url), mult, width });
                            }
                        }
                        for (const subtitle of subtitles) {
                            yield stores_1.filesToDownloadXPath.set(subtitle.path, { url: subtitle.url, namespace: '-' });
                        }
                        const _moduleDependencies = yield getModuleDependencies(nonPaginatedArticleId, mw, downloader);
                        for (const dep of _moduleDependencies.jsDependenciesList) {
                            jsModuleDependencies.add(dep);
                        }
                        for (const dep of _moduleDependencies.styleDependenciesList) {
                            cssModuleDependencies.add(dep);
                        }
                        jsConfigVars = jsConfigVars || _moduleDependencies.jsConfigVars;
                        let templatedDoc = yield templateArticle(articleDoc, _moduleDependencies, mw, dump, articleId, articleDetail, downloader.webp);
                        if (dump.customProcessor && dump.customProcessor.postProcessArticle) {
                            templatedDoc = yield dump.customProcessor.postProcessArticle(articleId, templatedDoc);
                        }
                        let outHtml = templatedDoc.documentElement.outerHTML;
                        if (dump.opts.minifyHtml) {
                            outHtml = html_minifier_1.default.minify(outHtml, {
                                removeComments: true,
                                conservativeCollapse: true,
                                collapseBooleanAttributes: true,
                                removeRedundantAttributes: true,
                                removeEmptyAttributes: true,
                                minifyCSS: true,
                            });
                        }
                        const finalHTML = `<!DOCTYPE html>\n` + outHtml;
                        const zimArticle = new libzim_1.ZimArticle({
                            url: articleId,
                            data: finalHTML,
                            ns: articleDetail.ns === 14 ? 'U' : 'A',
                            mimeType: 'text/html',
                            title: articleTitle,
                            shouldIndex: true,
                        });
                        zimCreator.addArticle(zimArticle);
                        dump.status.articles.success += 1;
                    }
                }
                catch (err) {
                    dump.status.articles.fail += 1;
                    Logger_1.default.error(`Error downloading article ${articleId}`);
                    if ((!err.response || err.response.status !== 404) && err.message !== const_1.DELETED_ARTICLE_ERROR) {
                        throw err;
                    }
                }
                if ((dump.status.articles.success + dump.status.articles.fail) % 10 === 0) {
                    const percentProgress = ((dump.status.articles.success + dump.status.articles.fail) / articlesTotal * 100).toFixed(1);
                    if (percentProgress !== prevPercentProgress) {
                        prevPercentProgress = percentProgress;
                        Logger_1.default.log(`Progress downloading articles [${dump.status.articles.success + dump.status.articles.fail}/${articlesTotal}] [${percentProgress}%]`);
                    }
                }
            }
        }));
        Logger_1.default.log(`Done with downloading a total of [${articlesTotal}] articles`);
        const jsConfigVarArticle = new libzim_1.ZimArticle({ url: _1.jsPath('jsConfigVars', config_1.config.output.dirs.mediawiki), data: jsConfigVars, ns: '-' });
        zimCreator.addArticle(jsConfigVarArticle);
        return {
            jsModuleDependencies,
            cssModuleDependencies,
        };
    });
}
exports.saveArticles = saveArticles;
function getModuleDependencies(articleId, mw, downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        /* These vars will store the list of js and css dependencies for
        the article we are downloading. */
        let jsConfigVars = '';
        let jsDependenciesList = [];
        let styleDependenciesList = [];
        const articleApiUrl = mw.articleApiUrl(articleId);
        const articleData = yield downloader.getJSON(articleApiUrl);
        /* Something went wrong in modules retrieval at app level (no HTTP error) */
        if (articleData.error) {
            Logger_1.default.warn(`Unable to retrieve js/css dependencies for article '${articleId}': ${articleData.error.code}`);
            /* If article is missing (for example because it just has been deleted) */
            if (articleData.error.code === 'missingtitle') {
                return { jsConfigVars, jsDependenciesList, styleDependenciesList };
            }
            /* Other kind of error (unsupported) */
            process.exit(1);
        }
        const { parse: { modules, modulescripts, modulestyles, headhtml } } = articleData;
        jsDependenciesList = genericJsModules.concat(modules, modulescripts).filter((a) => a);
        styleDependenciesList = [].concat(modules, modulestyles, genericCssModules).filter((a) => a);
        styleDependenciesList = styleDependenciesList.filter((oneStyleDep) => !_1.contains(config_1.config.filters.blackListCssModules, oneStyleDep));
        Logger_1.default.info(`Js dependencies of ${articleId} : ${jsDependenciesList}`);
        Logger_1.default.info(`Css dependencies of ${articleId} : ${styleDependenciesList}`);
        // Saving, as a js module, the jsconfigvars that are set in the header of a wikipedia page
        // the script below extracts the config with a regex executed on the page header returned from the api
        const scriptTags = domino.createDocument(`${headhtml['*']}</body></html>`).getElementsByTagName('script');
        const regex = /mw\.config\.set\(\{.*?\}\);/mg;
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < scriptTags.length; i += 1) {
            if (scriptTags[i].text.includes('mw.config.set')) {
                jsConfigVars = regex.exec(scriptTags[i].text)[0] || '';
            }
        }
        jsConfigVars = `(window.RLQ=window.RLQ||[]).push(function() {${jsConfigVars}});`;
        jsConfigVars = jsConfigVars.replace('nosuchaction', 'view'); // to replace the wgAction config that is set to 'nosuchaction' from api but should be 'view'
        return { jsConfigVars, jsDependenciesList, styleDependenciesList };
    });
}
function processArticleHtml(html, downloader, mw, dump, articleId) {
    return __awaiter(this, void 0, void 0, function* () {
        let mediaDependencies = [];
        let subtitles = [];
        let doc = domino.createDocument(html);
        const tmRet = yield treatMedias(doc, mw, dump, articleId, downloader);
        doc = tmRet.doc;
        mediaDependencies = mediaDependencies.concat(tmRet.mediaDependencies
            .filter((a) => a)
            .map((url) => {
            const path = _1.getMediaBase(url, false);
            return { url, path };
        }));
        subtitles = subtitles.concat(tmRet.subtitles
            .filter((a) => a)
            .map((url) => {
            const { title, lang } = QueryStringParser.parse(url);
            const path = `${title}-${lang}.vtt`;
            return { url, path };
        }));
        const ruRet = yield rewriteUrls(doc, articleId, downloader, mw, dump);
        doc = ruRet.doc;
        mediaDependencies = mediaDependencies.concat(ruRet.mediaDependencies
            .filter((a) => a)
            .map((url) => {
            const path = _1.getMediaBase(url, false);
            return { url, path };
        }));
        doc = applyOtherTreatments(doc, dump);
        return {
            articleDoc: doc,
            mediaDependencies,
            subtitles
        };
    });
}
function treatVideo(mw, dump, srcCache, articleId, videoEl, webp) {
    return __awaiter(this, void 0, void 0, function* () {
        /* Worth noting:
         - This function handles audio tags as well as video tags
         - Video tags are used for audio files too (as opposed to the audio tag)
         - When it's only audio, there will be a single OGG file
         - For video, we get multiple SOURCE tages with different resolutions */
        const mediaDependencies = [];
        const subtitles = [];
        /* Just delete video/audio element if the flavour requires it */
        if (dump.nopic || dump.novid || dump.nodet) {
            DOMUtils_1.default.deleteNode(videoEl);
            return { mediaDependencies, subtitles };
        }
        /* Firefox is not able to display correctly <video> nodes with a
         height < 40. In that case the controls are not displayed. */
        if (videoEl.getAttribute('height') && videoEl.getAttribute('height') < 40) {
            videoEl.setAttribute('height', '40');
        }
        /* Always show controls */
        videoEl.setAttribute('controls', '40');
        /* Choose best fiting resolution <source> video node */
        let videoSourceEls = Array.from(videoEl.children).filter((child) => child.tagName === 'SOURCE');
        const videoDisplayedWidth = Number(videoEl.getAttribute('width'));
        let bestWidthDiff = 424242;
        let chosenVideoSourceEl;
        videoSourceEls.forEach((videoSourceEl) => {
            // Ignore non-webm && non-audio sources
            const videoSourceType = videoSourceEl.getAttribute('type');
            if (!videoSourceEl.getAttribute('src').endsWith('.webm') &&
                !videoSourceType.startsWith('audio')) {
                DOMUtils_1.default.deleteNode(videoSourceEl);
                return;
            }
            // Handle audio content
            if (videoSourceType.startsWith('audio/ogg')) {
                chosenVideoSourceEl = videoSourceEl;
                return;
            }
            else if (videoSourceType.startsWith('audio')) {
                DOMUtils_1.default.deleteNode(videoSourceEl);
                return;
            }
            // If undefined displayed width, then take the best <source> resolution
            const videoSourceElWidth = Number(videoSourceEl.getAttribute('data-file-width') || videoSourceEl.getAttribute('data-width') || 0);
            if (!videoDisplayedWidth) {
                const chosenVideoSourceElWidth = chosenVideoSourceEl ?
                    chosenVideoSourceEl.getAttribute('data-file-width') || chosenVideoSourceEl.getAttribute('data-width') || 0 : 0;
                if (videoSourceElWidth > chosenVideoSourceElWidth ||
                    videoSourceElWidth == chosenVideoSourceElWidth && videoSourceEl.getAttribute('src').endsWith('.vp9.webm')) {
                    DOMUtils_1.default.deleteNode(chosenVideoSourceEl);
                    chosenVideoSourceEl = videoSourceEl;
                    return;
                }
            }
            // Otherwise, choose <source> with better (smaller) width diff
            else {
                const widthDiff = Number(videoSourceElWidth - videoDisplayedWidth);
                // If no source has been picked so far, just take this one
                if (!chosenVideoSourceEl) {
                    chosenVideoSourceEl = videoSourceEl;
                    bestWidthDiff = widthDiff;
                    return;
                }
                // Resolution of source is higher than displayed resolution
                else if (widthDiff >= 0) {
                    if (bestWidthDiff < 0 ||
                        widthDiff < bestWidthDiff ||
                        (widthDiff === bestWidthDiff && videoSourceEl.getAttribute('src').endsWith('.vp9.webm'))) {
                        DOMUtils_1.default.deleteNode(chosenVideoSourceEl);
                        chosenVideoSourceEl = videoSourceEl;
                        bestWidthDiff = widthDiff;
                        return;
                    }
                }
                // Resolution of source is smaller than displayed resolution
                else {
                    if (widthDiff > bestWidthDiff ||
                        (widthDiff === bestWidthDiff && videoSourceEl.getAttribute('src').endsWith('.vp9.webm'))) {
                        DOMUtils_1.default.deleteNode(chosenVideoSourceEl);
                        chosenVideoSourceEl = videoSourceEl;
                        bestWidthDiff = widthDiff;
                        return;
                    }
                }
            }
            // Delete all other nodes
            DOMUtils_1.default.deleteNode(videoSourceEl);
        });
        /* If no appropriate source video can be found, delete the video */
        if (!chosenVideoSourceEl) {
            Logger_1.default.warn(`Unable to find an appropriate video/audio source for '${videoEl.resource}'`);
            DOMUtils_1.default.deleteNode(videoEl);
            return { mediaDependencies, subtitles };
        }
        /* Remove useless 'resource' attribute */
        videoEl.removeAttribute('resource');
        /* Handle video poster */
        const posterUrl = videoEl.getAttribute('poster');
        if (posterUrl) {
            const videoPosterUrl = _1.getFullUrl(posterUrl, mw.baseUrl);
            const newVideoPosterUrl = misc_1.getRelativeFilePath(articleId, _1.getMediaBase(videoPosterUrl, true), 'I');
            if (posterUrl) {
                videoEl.setAttribute('poster', webp && misc_1.isWebpCandidateImageUrl(newVideoPosterUrl) ?
                    newVideoPosterUrl + '.webp' : newVideoPosterUrl);
            }
            videoEl.removeAttribute('resource');
            if (!srcCache.hasOwnProperty(videoPosterUrl)) {
                srcCache[videoPosterUrl] = true;
                mediaDependencies.push(videoPosterUrl);
            }
        }
        /* Download content, but avoid duplicate calls */
        const sourceUrl = _1.getFullUrl(chosenVideoSourceEl.getAttribute('src'), mw.baseUrl);
        if (!srcCache.hasOwnProperty(sourceUrl)) {
            srcCache[sourceUrl] = true;
            mediaDependencies.push(sourceUrl);
        }
        /* Set new URL for the video element */
        const fileBase = _1.getMediaBase(sourceUrl, true);
        chosenVideoSourceEl.setAttribute('src', misc_1.getRelativeFilePath(articleId, fileBase, 'I'));
        /* Scrape subtitle */
        for (const track of Array.from(videoEl.querySelectorAll('track'))) {
            subtitles.push(yield treatSubtitle(track, mw, articleId));
        }
        return { mediaDependencies, subtitles };
    });
}
exports.treatVideo = treatVideo;
function treatSubtitle(trackEle, mw, articleId) {
    return __awaiter(this, void 0, void 0, function* () {
        const subtitleSourceUrl = _1.getFullUrl(trackEle.getAttribute('src'), mw.baseUrl);
        const { title, lang } = QueryStringParser.parse(subtitleSourceUrl);
        // The source URL we get from Mediawiki article is in srt format, so we replace it to vtt which is standard subtitle trackformat for <track> src attribute.
        const vttFormatUrl = new URL(subtitleSourceUrl);
        vttFormatUrl.searchParams.set('trackformat', 'vtt');
        trackEle.setAttribute('src', `${misc_1.getRelativeFilePath(articleId, title, '-')}-${lang}.vtt`);
        return vttFormatUrl.href;
    });
}
exports.treatSubtitle = treatSubtitle;
function shouldKeepImage(dump, img) {
    const imageNodeClass = img.getAttribute('class') || '';
    const src = img.getAttribute('src');
    return (!dump.nopic
        || imageNodeClass.includes('mwe-math-fallback-image-inline')
        || img.getAttribute('typeof') === 'mw:Extension/math')
        && src
        && !src.includes('./Special:FilePath/');
}
function treatImage(mw, dump, srcCache, articleId, img, downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        const mediaDependencies = [];
        if (!shouldKeepImage(dump, img)) {
            DOMUtils_1.default.deleteNode(img);
            return { mediaDependencies };
        }
        /* Remove image link */
        const linkNode = img.parentNode;
        if (linkNode.tagName === 'A') {
            /* Check if the target is mirrored */
            const href = linkNode.getAttribute('href') || '';
            const title = mw.extractPageTitleFromHref(href);
            const keepLink = title && (yield isMirrored(title));
            /* Under certain condition it seems that this is possible
                             * to have parentNode == undefined, in this case this
                             * seems preferable to remove the whole link+content than
                             * keeping a wrong link. See for example this url
                             * http://parsoid.wmflabs.org/ko/%EC%9D%B4%ED%9C%98%EC%86%8C */
            if (!keepLink) {
                if (linkNode.parentNode) {
                    linkNode.parentNode.replaceChild(img, linkNode);
                }
                else {
                    DOMUtils_1.default.deleteNode(img);
                    return { mediaDependencies };
                }
            }
        }
        /* Rewrite image src attribute */
        const src = _1.getFullUrl(img.getAttribute('src'), mw.baseUrl);
        let newSrc;
        try {
            const resourceNamespace = 'I';
            const slashesInUrl = articleId.split('/').length - 1;
            const upStr = '../'.repeat(slashesInUrl + 1);
            newSrc = `${upStr}${resourceNamespace}/` + _1.getMediaBase(src, true);
            /* Download image, but avoid duplicate calls */
            if (!srcCache.hasOwnProperty(src)) {
                srcCache[src] = true;
                mediaDependencies.push(src);
            }
            /* Change image source attribute to point to the local image */
            img.setAttribute('src', downloader.webp && misc_1.isWebpCandidateImageUrl(newSrc) ?
                newSrc + '.webp' : newSrc);
            /* Remove useless 'resource' attribute */
            img.removeAttribute('resource');
            /* Remove srcset */
            img.removeAttribute('srcset');
        }
        catch (err) {
            DOMUtils_1.default.deleteNode(img);
        }
        /* Add lazy loading */
        img.setAttribute('loading', 'lazy');
        return { mediaDependencies };
    });
}
function isStillLinked(image) {
    return image && image.parentNode && image.parentNode.tagName === 'A';
}
function shouldKeepNode(dump, imageNode, image) {
    return !dump.nopic && imageNode && image;
}
function makeThumbDiv(dump, parsoidDoc, imageNode) {
    const imageNodeClass = imageNode.getAttribute('class') || '';
    let thumbDiv = parsoidDoc.createElement('div');
    thumbDiv.setAttribute('class', 'thumb');
    if (imageNodeClass.search('mw-halign-right') >= 0) {
        DOMUtils_1.default.appendToAttr(thumbDiv, 'class', 'tright');
    }
    else if (imageNodeClass.search('mw-halign-left') >= 0) {
        DOMUtils_1.default.appendToAttr(thumbDiv, 'class', 'tleft');
    }
    else if (imageNodeClass.search('mw-halign-center') >= 0) {
        DOMUtils_1.default.appendToAttr(thumbDiv, 'class', 'tnone');
        const centerDiv = parsoidDoc.createElement('center');
        centerDiv.appendChild(thumbDiv);
        thumbDiv = centerDiv;
    }
    else {
        const revAutoAlign = dump.mwMetaData.textDir === 'ltr' ? 'right' : 'left';
        DOMUtils_1.default.appendToAttr(thumbDiv, 'class', `t${revAutoAlign}`);
    }
    return thumbDiv;
}
function treatImageFrames(mw, dump, parsoidDoc, imageNode) {
    const image = imageNode.getElementsByTagName('img')[0] || imageNode.getElementsByTagName('video')[0];
    if (!shouldKeepNode(dump, imageNode, image)) {
        DOMUtils_1.default.deleteNode(imageNode);
        return;
    }
    const descriptions = imageNode.getElementsByTagName('figcaption');
    const description = descriptions.length > 0 ? descriptions[0] : undefined;
    const imageWidth = parseInt(image.getAttribute('width'), 10);
    const thumbDiv = makeThumbDiv(dump, parsoidDoc, imageNode);
    const thumbinnerDiv = parsoidDoc.createElement('div');
    thumbinnerDiv.setAttribute('class', 'thumbinner');
    thumbinnerDiv.setAttribute('style', `width:${imageWidth + 2}px`);
    const thumbcaptionDiv = parsoidDoc.createElement('div');
    thumbcaptionDiv.setAttribute('class', 'thumbcaption');
    const autoAlign = dump.mwMetaData.textDir === 'ltr' ? 'left' : 'right';
    thumbcaptionDiv.setAttribute('style', `text-align: ${autoAlign}`);
    if (description) {
        thumbcaptionDiv.innerHTML = description.innerHTML;
    }
    thumbinnerDiv.appendChild(isStillLinked(image) ? image.parentNode : image);
    thumbinnerDiv.appendChild(thumbcaptionDiv);
    thumbDiv.appendChild(thumbinnerDiv);
    imageNode.parentNode.replaceChild(thumbDiv, imageNode);
}
function treatMedias(parsoidDoc, mw, dump, articleId, downloader) {
    return __awaiter(this, void 0, void 0, function* () {
        let mediaDependencies = [];
        let subtitles = [];
        /* Clean/rewrite image tags */
        const imgs = Array.from(parsoidDoc.getElementsByTagName('img'));
        const videos = Array.from(parsoidDoc.querySelectorAll('video, audio'));
        const srcCache = {};
        for (const videoEl of videos) { // <video /> and <audio />
            const ret = yield treatVideo(mw, dump, srcCache, articleId, videoEl, downloader.webp);
            mediaDependencies = mediaDependencies.concat(ret.mediaDependencies);
            subtitles = subtitles.concat(ret.subtitles);
        }
        for (const imgEl of imgs) {
            const ret = yield treatImage(mw, dump, srcCache, articleId, imgEl, downloader);
            mediaDependencies = mediaDependencies.concat(ret.mediaDependencies);
        }
        /* Improve image frames */
        const figures = parsoidDoc.getElementsByTagName('figure');
        const spans = parsoidDoc.querySelectorAll('span[typeof=mw:Image/Frameless]');
        const imageNodes = Array.prototype.slice.call(figures).concat(Array.prototype.slice.call(spans));
        for (const imageNode of imageNodes) {
            treatImageFrames(mw, dump, parsoidDoc, imageNode);
        }
        return { doc: parsoidDoc, mediaDependencies, subtitles };
    });
}
exports.treatMedias = treatMedias;
function rewriteUrls(parsoidDoc, articleId, downloader, mw, dump) {
    return __awaiter(this, void 0, void 0, function* () {
        let mediaDependencies = [];
        /* Go through all links */
        const as = parsoidDoc.getElementsByTagName('a');
        const areas = parsoidDoc.getElementsByTagName('area');
        const linkNodes = Array.prototype.slice.call(as).concat(Array.prototype.slice.call(areas));
        yield p_map_1.default(linkNodes, (linkNode) => __awaiter(this, void 0, void 0, function* () {
            const { mediaDependencies: mediaDeps } = yield rewriteUrls_1.rewriteUrl(articleId, mw, dump, linkNode);
            mediaDependencies = mediaDependencies.concat(mediaDeps);
        }), { concurrency: downloader.speed });
        return { doc: parsoidDoc, mediaDependencies };
    });
}
function applyOtherTreatments(parsoidDoc, dump) {
    const filtersConfig = config_1.config.filters;
    /* Don't need <link> and <input> tags */
    const nodesToDelete = [{ tag: 'link' }, { tag: 'input' }];
    /* Remove "map" tags if necessary */
    if (dump.nopic) {
        nodesToDelete.push({ tag: 'map' });
    }
    /* Remove useless DOM nodes without children */
    function emptyChildFilter(n) {
        return !n.innerHTML;
    }
    nodesToDelete.push({ tag: 'li', filter: emptyChildFilter });
    nodesToDelete.push({ tag: 'span', filter: emptyChildFilter });
    /* Remove gallery boxes if pics need stripping of if it doesn't have thumbs */
    nodesToDelete.push({
        class: 'gallerybox',
        filter(n) {
            return !n.getElementsByTagName('img').length
                && !n.getElementsByTagName('audio').length
                && !n.getElementsByTagName('video').length;
        },
    });
    nodesToDelete.push({
        class: 'gallery',
        filter(n) {
            return !n.getElementsByClassName('gallerybox').length;
        },
    });
    /* Remove element with black listed CSS classes */
    filtersConfig.cssClassBlackList.forEach((classname) => {
        nodesToDelete.push({ class: classname });
    });
    if (dump.nodet) {
        filtersConfig.nodetCssClassBlackList.forEach((classname) => {
            nodesToDelete.push({ class: classname });
        });
    }
    /* Remove element with black listed CSS classes and no link */
    filtersConfig.cssClassBlackListIfNoLink.forEach((classname) => {
        nodesToDelete.push({
            class: classname,
            filter(n) {
                return n.getElementsByTagName('a').length === 0;
            },
        });
    });
    /* Delete them all */
    for (const t of nodesToDelete) {
        let nodes;
        if (t.tag) {
            nodes = parsoidDoc.getElementsByTagName(t.tag);
        }
        else if (t.class) {
            nodes = parsoidDoc.getElementsByClassName(t.class);
        }
        else {
            return; /* throw error? */
        }
        for (const node of Array.from(nodes)) {
            if (!t.filter || t.filter(node)) {
                DOMUtils_1.default.deleteNode(node);
            }
        }
    }
    /* Go through all reference calls */
    const spans = Array.from(parsoidDoc.getElementsByTagName('span'));
    for (const span of spans) {
        const rel = span.getAttribute('rel');
        if (rel === 'dc:references') {
            const sup = parsoidDoc.createElement('sup');
            if (span.innerHTML) {
                sup.id = span.id;
                sup.innerHTML = span.innerHTML;
                span.parentNode.replaceChild(sup, span);
            }
            else {
                DOMUtils_1.default.deleteNode(span);
            }
        }
    }
    /* Remove element with id in the blacklist */
    filtersConfig.idBlackList.forEach((id) => {
        const node = parsoidDoc.getElementById(id);
        if (node) {
            DOMUtils_1.default.deleteNode(node);
        }
    });
    /* Force display of element with that CSS class */
    filtersConfig.cssClassDisplayList.map((classname) => {
        const nodes = Array.from(parsoidDoc.getElementsByClassName(classname));
        for (const node of nodes) {
            node.style.removeProperty('display');
        }
    });
    /* Remove empty paragraphs */
    if (!dump.opts.keepEmptyParagraphs) {
        // Mobile view === details
        // Desktop view === section
        const sections = Array.from(parsoidDoc.querySelectorAll('details, section'));
        for (const section of sections) {
            if (section.children.length === Array.from(section.children).filter((child) => { return child.matches('summary'); }).length) {
                DOMUtils_1.default.deleteNode(section);
            }
        }
    }
    /* Clean the DOM of all uncessary code */
    const allNodes = Array.from(parsoidDoc.getElementsByTagName('*'));
    for (const node of allNodes) {
        node.removeAttribute('data-parsoid');
        node.removeAttribute('typeof');
        node.removeAttribute('about');
        node.removeAttribute('data-mw');
        if (node.getAttribute('rel') && node.getAttribute('rel').substr(0, 3) === 'mw:') {
            node.removeAttribute('rel');
        }
        /* Remove a few images Parsoid attributes */
        else if (node.getAttribute('img')) {
            node.removeAttribute('data-file-width');
            node.removeAttribute('data-file-height');
            node.removeAttribute('data-file-type');
        }
        /* Remove a few css calls */
        filtersConfig.cssClassCallsBlackList.map((classname) => {
            if (node.getAttribute('class')) {
                node.setAttribute('class', node.getAttribute('class').replace(classname, ''));
            }
        });
    }
    const kartographerMaplinkNodes = Array.from(parsoidDoc.querySelectorAll('.mw-kartographer-maplink'))
        .filter((n) => !!n.textContent);
    for (const node of kartographerMaplinkNodes) {
        node.textContent = '🌍';
    }
    return parsoidDoc;
}
exports.applyOtherTreatments = applyOtherTreatments;
function templateArticle(parsoidDoc, moduleDependencies, mw, dump, articleId, articleDetail, webp) {
    return __awaiter(this, void 0, void 0, function* () {
        const { jsConfigVars, jsDependenciesList, styleDependenciesList, } = moduleDependencies;
        const htmlTemplateDoc = domino.createDocument(Templates_1.htmlTemplateCode(articleId)
            .replace('__ARTICLE_CANONICAL_LINK__', _1.genCanonicalLink(config_1.config, mw.webUrl.href, articleId))
            .replace('__ARTICLE_CONFIGVARS_LIST__', jsConfigVars !== '' ? _1.genHeaderScript(config_1.config, 'jsConfigVars', articleId, config_1.config.output.dirs.mediawiki) : '')
            .replace('__ARTICLE_JS_LIST__', jsDependenciesList.length !== 0
            ? jsDependenciesList.map((oneJsDep) => _1.genHeaderScript(config_1.config, oneJsDep, articleId, config_1.config.output.dirs.mediawiki)).join('\n')
            : '')
            .replace('__ARTICLE_CSS_LIST__', styleDependenciesList.length !== 0
            ? styleDependenciesList.map((oneCssDep) => _1.genHeaderCSSLink(config_1.config, oneCssDep, articleId, config_1.config.output.dirs.mediawiki)).join('\n')
            : '')
            .replace('__ARTICLE_ID__', `<script>let articleId = '${articleId}'</script>`));
        /* Create final document by merging template and parsoid documents */
        htmlTemplateDoc.getElementById('mw-content-text').style.setProperty('direction', dump.mwMetaData.textDir);
        htmlTemplateDoc.getElementById('mw-content-text').innerHTML = parsoidDoc.getElementsByTagName('body')[0].innerHTML;
        /* Title */
        htmlTemplateDoc.getElementsByTagName('title')[0].innerHTML = htmlTemplateDoc.getElementById('title_0')
            ? htmlTemplateDoc.getElementById('title_0').textContent
            : articleId.replace(/_/g, ' ');
        DOMUtils_1.default.deleteNode(htmlTemplateDoc.getElementById('titleHeading'));
        /* Subpage */
        if (isSubpage(articleId, mw) && !dump.isMainPage(articleId)) {
            const headingNode = htmlTemplateDoc.getElementById('mw-content-text');
            const subpagesNode = htmlTemplateDoc.createElement('span');
            const parents = articleId.split('/');
            parents.pop();
            let subpages = '';
            yield Promise.all(parents.map((parent) => __awaiter(this, void 0, void 0, function* () {
                const label = parent.replace(/_/g, ' ');
                const isParentMirrored = yield isMirrored(`${articleId.split(parent)[0]}${parent}`);
                subpages
                    += `&lt; ${isParentMirrored
                        ? `<a href="${'../'.repeat(parents.length)}${misc_1.encodeArticleIdForZimHtmlUrl(`${articleId.split(parent)[0]}${parent}`)}" title="${label}">`
                        : ''}${label}${isParentMirrored ? '</a> ' : ' '}`;
            })));
            subpagesNode.innerHTML = subpages;
            subpagesNode.setAttribute('class', 'subpages');
            headingNode.parentNode.insertBefore(subpagesNode, headingNode);
        }
        /* Set footer */
        const div = htmlTemplateDoc.createElement('div');
        /* Revision date */
        const date = new Date(articleDetail.timestamp);
        const lastEditedOnString = date ?
            misc_1.interpolateTranslationString(dump.strings.LAST_EDITED_ON, {
                date: date.toISOString().substring(0, 10),
            }) : null;
        const creatorLink = `<a class="external text" ` +
            `${lastEditedOnString ? `title="${lastEditedOnString}"` : ''} ` +
            `href="${mw.webUrl.href}?title=${encodeURIComponent(articleId)}&oldid=${articleDetail.revisionId}">` +
            `${dump.mwMetaData.creator}</a>`;
        const licenseLink = `<a class="external text" href="https://creativecommons.org/licenses/by-sa/4.0/">${dump.strings.LICENSE_NAME}</a>`;
        div.innerHTML = Templates_1.footerTemplate({
            disclaimer: misc_1.interpolateTranslationString(dump.strings.DISCLAIMER, {
                creator: creatorLink,
                license: licenseLink,
            }),
            strings: dump.strings,
        });
        htmlTemplateDoc.getElementById('mw-content-text').appendChild(div);
        addNoIndexCommentToElement(div);
        /* Geo-coordinates */
        if (articleDetail.coordinates) {
            const geoCoordinates = articleDetail.coordinates;
            const metaNode = htmlTemplateDoc.createElement('meta');
            metaNode.name = 'geo.position';
            metaNode.content = geoCoordinates;
            htmlTemplateDoc.getElementsByTagName('head')[0].appendChild(metaNode);
        }
        return htmlTemplateDoc;
    });
}
function addNoIndexCommentToElement(element) {
    const slices = element.parentElement.innerHTML.split(element.outerHTML);
    element.parentElement.innerHTML = `${slices[0]}<!--htdig_noindex-->${element.outerHTML}<!--/htdig_noindex-->${slices[1]}`;
}
function isSubpage(id, mw) {
    if (id && id.indexOf('/') >= 0) {
        const namespace = id.indexOf(':') >= 0 ? id.substring(0, id.indexOf(':')) : '';
        const ns = mw.namespaces[namespace]; // namespace already defined
        if (ns !== undefined) {
            return ns.allowedSubpages;
        }
    }
    return false;
}
function isMirrored(id) {
    return stores_1.articleDetailXId.get(id);
}
exports.isMirrored = isMirrored;
//# sourceMappingURL=saveArticles.js.map